@@ -1,6 +1,6 @@
 /*
 
-  $Id: cfgreader.c,v 1.78 2007/05/29 19:26:40 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -23,14 +23,14 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
   Copyright (C) 1999      Tim Potter
-  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Janik ml.
   Copyright (C) 2000      Jan Derfinak
   Copyright (C) 2001      Jan Kratochvil
-  Copyright (C) 2001-2005 Pawel Kot
+  Copyright (C) 2001-2009 Pawel Kot
   Copyright (C) 2002-2004 BORBELY Zoltan
   Copyright (C) 2005      Bastien Nocera
 
-  Config file (/etc/gnokiirc and ~/.gnokiirc) reader.
+  Config reader.
 
   Modified from code by Tim Potter.
 
@@ -52,6 +52,293 @@
 GNOKII_API struct gn_cfg_header *gn_cfg_info;
 static gn_config gn_config_default, gn_config_global;
 
+/* Handy macros */
+#define FREE(x)	do { if (x) free(x); (x) = NULL; } while (0)
+
+/*
+ * Function to dump configuration pointed by @config
+ * @config:	configuration to be dumped
+ */
+static void cfg_dump(struct gn_cfg_header *config)
+{
+	struct gn_cfg_header *hdr = config;
+
+	dprintf("Dumping configuration.\n");
+	/* Loop over sections */
+	while (hdr) {
+		struct gn_cfg_entry *entry = hdr->entries;
+
+		dprintf("[%s]\n", hdr->section);
+		/* Loop over entries */
+		while (entry) {
+			dprintf("%s = %s\n", entry->key, entry->value);
+			entry = entry->next;
+		}
+		hdr = hdr->next;
+	}
+}
+
+/*
+ * Function to allocate configuration header structure.
+ * Structure is appended to @config.
+ * @config:	existing config structure
+ * @name:	section name; default (if name==NULL) is "global"
+ */
+static struct gn_cfg_header *cfg_header_allocate(struct gn_cfg_header *config, char *name)
+{
+	struct gn_cfg_header *hdr;
+
+	/*
+	 * Allocate memory for the header structure and assign name.
+	 * Default name is "global".
+	 */
+	hdr = calloc(sizeof(struct gn_cfg_header), 1);
+	if (!hdr) {
+		dprintf("Failed to allocate gn_cfg_header.\n");
+		return NULL;
+	}
+	if (name)
+		hdr->section = strdup(name);
+	else
+		hdr->section = strdup("global");
+	if (!hdr->section) {
+		dprintf("Failed to assign a name to gn_cfg_header.\n");
+		free(hdr);
+		return NULL;
+	}
+
+	/* Add to cfg_info list */
+	hdr->prev = config;
+	if (config)
+		config->next = hdr;
+
+	dprintf("Adding new section %s\n", hdr->section);
+
+	return hdr;
+}
+
+/*
+ * Function to find config section identified by @name. If name is not given
+ * "global" section is returned.
+ * @config:	pointer to the configuration structure
+ * @name:	section name
+ */
+static struct gn_cfg_header *cfg_header_get(struct gn_cfg_header *config, char *name)
+{
+	struct gn_cfg_header *hdr = config;
+
+	if (!config)
+		return NULL;
+
+	if (!name)
+		name = "global";
+
+	while (hdr) {
+		if (!strcmp(name, hdr->section))
+			return hdr;
+		hdr = hdr->next;
+	}
+	return NULL;
+}
+
+/*
+ * Function to assign a variable in the given config section.
+ * @config:	given configuration section
+ * @name:	variable (key) name
+ * @value:	variable (key) value
+ * @overwrite:	overwrite existing keys?
+ */
+static struct gn_cfg_header *cfg_variable_set(struct gn_cfg_header *config, char *name, char *value, int overwrite)
+{
+	struct gn_cfg_entry *oldentry, *entry;
+
+	if (!name || !value) {
+		dprintf("Neither name nor value can be NULL.\n");
+		return NULL;
+	}
+
+	/* Find a place in the section, where to add the new entry. */
+	oldentry = config->entries;
+	while (oldentry) {
+		if (!strcmp(name, oldentry->key))
+			break;
+		oldentry = oldentry->next;
+	}
+
+	/* If found and instructed not to overwrite, return */
+	if (oldentry && !overwrite) {
+		dprintf("Key %s already exists in section %s\n", name, config->section);
+		return NULL;
+	}
+
+	/* Allocate new entry */
+	entry = calloc(sizeof(struct gn_cfg_entry), 1);
+	if (!entry) {
+		dprintf("Failed to allocate gn_cfg_entry.\n");
+		return NULL;
+	}
+	entry->key = strdup(name);
+	entry->value = strdup(value);
+	if (!entry->key || !entry->value) {
+		dprintf("Failed to allocate key/value for the entry.\n");
+		FREE(entry->key);
+		FREE(entry->value);
+		free(entry);
+		return NULL;
+	}
+
+	/* Add to existing variables in section. */
+	entry->next = config->entries;
+	if (config->entries)
+		config->entries->prev = entry;
+	config->entries = entry;
+
+	/* Remove the old entry */
+	if (oldentry) {
+		if (oldentry->next)
+			oldentry->next->prev = oldentry->prev;
+		if (oldentry->prev)
+			oldentry->prev->next = oldentry->next;
+		free(oldentry->key);
+		free(oldentry->value);
+		free(oldentry);
+	}
+
+	dprintf("Added %s/%s to section %s.\n", entry->key, entry->value, config->section);
+
+	return config;
+}
+
+/*
+ * Function to assign a variable in the given section. If the section is not
+ * found in the config structure, new section is created.
+ * @config:	configuration
+ * @section:	section name
+ * @name:	variable (key) name
+ * @value:	variable (key) value
+ * @overwrite:	overwrite existing keys?
+ */
+GNOKII_API struct gn_cfg_header *gn_cfg_variable_set(struct gn_cfg_header *config, char *section, char *name, char *value, int overwrite)
+{
+	struct gn_cfg_header *hdr;
+
+	/* Find the appropriate section */
+	hdr = cfg_header_get(config, section);
+	if (!hdr)
+		hdr = cfg_header_allocate(config, section);
+	if (!hdr) {
+		dprintf("Failed to set variable (%s %s %s).\n", section, name, value);
+		return NULL;
+	}
+
+	hdr = cfg_variable_set(hdr, name, value, overwrite);
+
+	return hdr;
+}
+
+/*
+ * Function to create generic configuration section. Basic config consists of model,
+ * connection and port variables. If no section name is given, global one is used.
+ * Please note that valid section names for phone config are: global and phone_*.
+ * @section:	optional section name
+ * @model:	model to be set
+ * @connection:	connection to be set
+ * @port:	port to be set
+ */
+GNOKII_API struct gn_cfg_header *gn_cfg_section_create(char *section, char *model, char *connection, char *port)
+{
+	char *sname;
+	struct gn_cfg_header *hdr = NULL, *config = NULL;
+
+	if (!model | !connection | !port) {
+		dprintf("Neither model nor connection nor port can be NULL.\n");
+		return NULL;
+	}
+
+	sname = (section ? section : "global");
+
+	config = cfg_header_allocate(NULL, sname);
+	if (!config) {
+		dprintf("Failed to create config.\n");
+		return NULL;
+	}
+	hdr = gn_cfg_variable_set(config, sname, "model", model, 1);
+	if (!hdr) {
+		dprintf("Failed to create config.\n");
+		free(config);
+		return NULL;
+	}
+	hdr = gn_cfg_variable_set(config, sname, "connection", connection, 1);
+	if (!hdr) {
+		dprintf("Failed to create config.\n");
+		free(config);
+		return NULL;
+	}
+	hdr = gn_cfg_variable_set(config, sname, "port", port, 1);
+	if (!hdr) {
+		dprintf("Failed to create config.\n");
+		free(config);
+		return NULL;
+	}
+
+	return config;
+}
+
+/*
+ * Function to create generic config. Config is assigned to gn_cfg_info structure.
+ * @model:	model to be set
+ * @connection:	connection to be set
+ * @port:	port to be set
+ */
+GNOKII_API struct gn_cfg_header *gn_cfg_generic_create(char *model, char *connection, char *port)
+{
+	struct gn_cfg_header *config;
+
+	config = gn_cfg_section_create(NULL, model, connection, port);
+	if (!config)
+		return NULL;
+
+	/* Dump config */
+	cfg_dump(config);
+
+	/* Assign to gn_cfg_info. libgnokii needs it so far. */
+	gn_cfg_info = config;
+	return config;
+}
+
+/*
+ * Function to create basic config for bluetooth setup. Config is assigned to gn_cfg_info structure.
+ * @model:	model to be set
+ * @connection:	connection to be set
+ * @port:	port to be set
+ */
+GNOKII_API struct gn_cfg_header *gn_cfg_bluetooth_create(char *model, char *btmac, char *rfchannel)
+{
+	struct gn_cfg_header *hdr, *config;
+
+	if (!model | !btmac | !rfchannel) {
+		dprintf("Neither model nor Bluetooth mac address nor rfcomm channel can be NULL.\n");
+		return NULL;
+	}
+
+	config = gn_cfg_section_create(NULL, model, "bluetooth", btmac);
+	if (!config)
+		return NULL;
+	hdr = gn_cfg_variable_set(config, "global", "rfcomm_channel", rfchannel, 1);
+	if (!hdr) {
+		dprintf("Failed to create config.\n");
+		free(config);
+		return NULL;
+	}
+
+	/* Dump config */
+	cfg_dump(config);
+
+	/* Assign to gn_cfg_info. libgnokii needs it so far. */
+	gn_cfg_info = config;
+	return config;
+}
+
 struct gn_cfg_header *cfg_memory_read(const char **lines)
 {
 	char *line, *buf;
@@ -73,10 +360,10 @@ struct gn_cfg_header *cfg_memory_read(const char **lines)
 		buf = line;
 
 		/* Strip leading, trailing whitespace */
-		while(isspace((int) *line))
+		while (isspace((int) *line))
 			line++;
 
-		while((strlen(line) > 0) && isspace((int) line[strlen(line) - 1]))
+		while ((strlen(line) > 0) && isspace((int) line[strlen(line) - 1]))
 			line[strlen(line) - 1] = '\0';
 
 		/* Ignore blank lines and comments */
@@ -146,7 +433,7 @@ struct gn_cfg_header *cfg_memory_read(const char **lines)
 
 			entry->value = strdup(value);
 
-			while((strlen(line) > 0) && isspace((int) line[strlen(line) - 1])) {
+			while ((strlen(line) > 0) && isspace((int) line[strlen(line) - 1])) {
 				line[strlen(line) - 1] = '\0';  /* Remove trailing white */
 			}
 
@@ -171,8 +458,8 @@ struct gn_cfg_header *cfg_memory_read(const char **lines)
 			continue;
 		}
 
-			/* Line not part of any heading */
-		fprintf(stderr, _("Orphaned line: %s\n"), line);
+		/* Line not part of any heading */
+		fprintf(stderr, _("Orphaned line: %s\nIf in doubt place this line into [global] section.\n"), line);
 
 		free(buf);
 	}
@@ -248,8 +535,10 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 	} while (ret > 0);
 
 	fclose(handle);
-	lines = realloc(lines, read + 1);
-	lines[read] = '\0';
+	/* Make sure there's '\n' after the last line and NULL-terminate. */
+	lines = realloc(lines, read + 2);
+	lines[read] = '\n';
+	lines[read+1] = '\0';
 
 	/* Now split the lines */
 	split_lines = NULL;
@@ -258,7 +547,15 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 	copied = 0;
 
 	while ((pos = strchr(line_begin, '\n')) && copied < read) {
-		char *buf;
+		char *buf, *hash, *tmp;
+		int nonempty = 0;
+
+		/* Strip comments from the current line */
+		hash = strchr(line_begin, '#');
+		if (hash && (hash < pos || !pos))
+			line_begin[hash-line_begin] = '\0';
+		else
+			hash = NULL;
 
 		if (!pos) {
 			line_end = lines + read;
@@ -266,13 +563,23 @@ struct gn_cfg_header *cfg_file_read(const char *filename)
 			line_end = pos;
 		}
 
-		num_lines++;
-		buf = malloc(line_end - line_begin + 1);
-		strncpy(buf, line_begin, line_end - line_begin);
-		buf[line_end - line_begin] = 0;
-		split_lines = realloc(split_lines,
-				(num_lines + 1) * sizeof(char*));
-		split_lines[num_lines - 1] = buf;
+		/* skip empty lines */
+		tmp = line_begin;
+		nonempty = 0;
+		while (*tmp && tmp < line_end && !nonempty) {
+			if (!isspace(*tmp))
+				nonempty = 1;
+			tmp++;
+		}
+
+		if (nonempty) {
+			num_lines++;
+			buf = malloc((hash ? hash : line_end) - line_begin + 1);
+			snprintf(buf, (hash ? hash : line_end) - line_begin + 1, "%s", line_begin);
+			split_lines = realloc(split_lines,
+					(num_lines + 1) * sizeof(char*));
+			split_lines[num_lines - 1] = buf;
+		}
 
 		if (pos) {
 			copied += (line_end + 1 - line_begin);
@@ -342,6 +649,19 @@ GNOKII_API char *gn_cfg_get(struct gn_cfg_header *cfg, const char *section, cons
 	return NULL;
 }
 
+int cfg_section_exists(struct gn_cfg_header *cfg, const char *section)
+{
+	struct gn_cfg_header *h;
+
+	if (!cfg || !section)
+		return false;
+	/* Search for section name */
+	for (h = cfg; h != NULL; h = h->next)
+		if (!strcmp(section, h->section))
+			return true;
+	return false;
+}
+
 /*
  * Return all the entries of the given section.
  */
@@ -397,34 +717,42 @@ char *cfg_set(struct gn_cfg_header *cfg, const char *section, const char *key,
 	return NULL;
 }
 
-static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_config *def)
+static gn_error cfg_psection_load(gn_config *cfg, const char *section, const gn_config *def)
 {
 	const char *val;
 	char ch;
 
 	memset(cfg, '\0', sizeof(gn_config));
 
+	if (!cfg_section_exists(gn_cfg_info, section)) {
+		fprintf(stderr, _("No %s section in the config file.\n"), section);
+		return GN_ERR_NOPHONE;
+	}
+
 	/* You need to specify at least model and port in the phone section */
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "model")))
-		/* strcpy(cfg->model, def->model); */
-		return false;
-	else
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "model"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "model");
+		return GN_ERR_NOMODEL;
+	} else
 		snprintf(cfg->model, sizeof(cfg->model), "%s", val);
 
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "port")))
-		/* strcpy(cfg->port_device, def->port_device); */
-		return false;
-	else
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "port"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "port");
+		return GN_ERR_NOPORT;
+	} else
 		snprintf(cfg->port_device, sizeof(cfg->port_device), "%s", val);
 
-	if (!(val = gn_cfg_get(gn_cfg_info, section, "connection")))
-		cfg->connection_type = def->connection_type;
-	else {
+	if (!(val = gn_cfg_get(gn_cfg_info, section, "connection"))) {
+		fprintf(stderr, _("You need to define '%s' in the config file.\n"), "connection");
+		return GN_ERR_NOCONNECTION;
+	} else {
 		cfg->connection_type = gn_get_connectiontype(val);
 		if (cfg->connection_type == GN_CT_NONE) {
 			fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), section, "connection", val);
-			return false;
+			return GN_ERR_NOCONNECTION;
 		}
+		if (!strcmp("pcsc", val))
+			fprintf(stderr, "WARNING: %s=%s is deprecated and will soon be removed. Use %s=%s instead.\n", "connection", val, "connection", "libpcsclite");
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "initlength")))
@@ -487,12 +815,12 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 	}
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "connect_script")))
-		strcpy(cfg->connect_script, def->connect_script);
+		snprintf(cfg->connect_script, sizeof(cfg->connect_script), "%s", def->connect_script);
 	else
 		snprintf(cfg->connect_script, sizeof(cfg->connect_script), "%s", val);
 
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "disconnect_script")))
-		strcpy(cfg->disconnect_script, def->disconnect_script);
+		snprintf(cfg->disconnect_script, sizeof(cfg->disconnect_script), "%s", def->disconnect_script);
 	else
 		snprintf(cfg->disconnect_script, sizeof(cfg->disconnect_script), "%s", val);
 
@@ -515,7 +843,7 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 	/* There is no global setting. You need to set irda_string
 	 * in each section if you want it working */
 	if (!(val = gn_cfg_get(gn_cfg_info, section, "irda_string")))
-		strcpy(cfg->irda_string, "");
+		cfg->irda_string[0] = 0;
 	else {
 		snprintf(cfg->irda_string, sizeof(cfg->irda_string), "%s", val);
 		dprintf("Setting irda_string in section %s to %s\n", section, cfg->irda_string);
@@ -534,7 +862,7 @@ static bool cfg_psection_load(gn_config *cfg, const char *section, const gn_conf
 		cfg->use_locking = def->use_locking;
 	}
 
-	return true;
+	return GN_ERR_NONE;
 }
 
 static bool cfg_get_log_target(gn_log_target *t, const char *opt)
@@ -550,7 +878,7 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 		*t = GN_LOG_T_STDERR;
 	else {
 		fprintf(stderr, _("Unsupported [%s] %s value \"%s\"\n"), "logging", opt, val);
-		fprintf(stderr, _("Use either \"%s\" or \"%s\".\n"), "yes", "no");
+		fprintf(stderr, _("Use either \"%s\" or \"%s\".\n"), "off", "on");
 		fprintf(stderr, _("Assuming: %s\n"), "off");
 		*t = GN_LOG_T_NONE;
 	}
@@ -558,66 +886,189 @@ static bool cfg_get_log_target(gn_log_target *t, const char *opt)
 	return true;
 }
 
-#define MAX_PATH_LEN 200
-GNOKII_API gn_error gn_cfg_read_default()
-{
-	char *homedir;
-	char rcfile[MAX_PATH_LEN];
+#define MAX_PATH_LEN 255
+
+#define CHECK_SIZE()	if (*retval >= size) { \
+	size *= 2; \
+	config_file_locations = realloc(config_file_locations, size); \
+}
+
+/*
+ * get_locations() returns the list of possible config file locations that
+ * may be used on the given system.  It returns the array of paths to the
+ * config files.  @retval parameter denotes how many locations are returned. 
+ * The resulting array needs to be freed afterwards.
+ */ 
 #ifdef WIN32
-	char globalrc[MAX_PATH_LEN];
-	char *systemroot;
+/* Windows version */
+static char **get_locations(int *retval)
+{
+	char **config_file_locations;
+	char *appdata, *homedrive, *homepath, *systemroot; /* env variables */
+	int size = 3; /* default size for config_file_locations */
+	char path[MAX_PATH_LEN];
 
+	*retval = size;
+
+	config_file_locations = calloc(size, sizeof(char *));
+
+	appdata = getenv("APPDATA");
+	homedrive = getenv("HOMEDRIVE");
+	homepath = getenv("HOMEPATH");
 	systemroot = getenv("SYSTEMROOT");
-	strncpy(globalrc, systemroot ? systemroot : "", MAX_PATH_LEN);
-	strncat(globalrc, "\\gnokiirc", MAX_PATH_LEN);
+	/* 1. %APPDATA%\gnokii\config */
+	snprintf(path, MAX_PATH_LEN, "%s\\gnokii\\config", appdata);
+	config_file_locations[0] = strdup(path);
+	/* old gnokii behaviour */
+	/* 2. %HOMEDRIVE%\%HOMEPATH%\_gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s\\%s\\_gnokiirc", homedrive, homepath);
+	config_file_locations[1] = strdup(path);
+	/* 3. %SYSTEMROOT%\gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s\\gnokiirc", systemroot);
+	config_file_locations[2] = strdup(path);
 
-	homedir = getenv("HOMEDRIVE");
-	strncpy(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
-	homedir = getenv("HOMEPATH");
-	strncat(rcfile, homedir ? homedir : "", MAX_PATH_LEN);
-	strncat(rcfile, "\\_gnokiirc", MAX_PATH_LEN);
-#else
-	const char globalrc[] = "/etc/gnokiirc";
+	return config_file_locations;
+}
+#else /* WIN32 */
+#  ifdef __MACH__
+/* Mac OS X version */
+static char **get_locations(int *retval)
+{
+	char **config_file_locations;
+	char *home; /* env variables */
+	int size = 3; /* default size for config_file_locations */
+	char path[MAX_PATH_LEN];
 
-	homedir = getenv("HOME");
-	if (homedir) strncpy(rcfile, homedir, MAX_PATH_LEN);
-	strncat(rcfile, "/.gnokiirc", MAX_PATH_LEN);
-#endif
+	*retval = size;
 
-	/* Try opening .gnokirc from users home directory first */
-	if (gn_cfg_file_read(rcfile)) {
-		/* It failed so try for /etc/gnokiirc */
-		if (gn_cfg_file_read(globalrc)) {
-			/* That failed too so exit */
-			fprintf(stderr, _("Couldn't open %s or %s.\n"), rcfile, globalrc);
-			return GN_ERR_NOCONFIG;
+	config_file_locations = calloc(size, sizeof(char *));
+
+	home = getenv("HOME");
+	/* 1. $HOME/Library/Preferences/gnokii/config */
+	snprintf(path, MAX_PATH_LEN, "%s/Library/Preferences/gnokii/config", home);
+	config_file_locations[0] = strdup(path);
+	/* old gnokii behaviour */
+	/* 2. $HOME/.gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s/.gnokiirc", home);
+	config_file_locations[1] = strdup(path);
+	/* 3. /etc/gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "/etc/gnokiirc");
+	config_file_locations[2] = strdup(path);
+
+	return config_file_locations;
+}
+#  else /* !__MACH__ */
+/* freedesktop.org compliancy: http://standards.freedesktop.org/basedir-spec/latest/ar01s03.html */
+#define XDG_CONFIG_HOME	"/.config"	/* $HOME/.config */
+#define XDG_CONFIG_DIRS "/etc/xdg"
+static char **get_locations(int *retval)
+{
+	char **config_file_locations;
+	char *xdg_config_home, *xdg_config_dirs, *home; /* env variables */
+	char **xdg_config_dir;
+	char *aux, *dirs;
+	int j, i = 0;
+	char path[MAX_PATH_LEN];
+	int size = 8; /* default size for config_file_locations */
+	int xcd_size = 4; /* default size for xdg_config_dir - number of elements */
+	int free_xdg_config_home = 0;
+
+	*retval = 0;
+	config_file_locations = calloc(size, sizeof(char *));
+
+	/* First, let's get all env variables we will need */
+	home = getenv("HOME");
+
+	xdg_config_home = getenv("XDG_CONFIG_HOME");
+	if (!xdg_config_home) {
+		xdg_config_home = calloc(MAX_PATH_LEN, sizeof(char));
+		free_xdg_config_home = 1;
+		sprintf(xdg_config_home, "%s%s", home, XDG_CONFIG_HOME);
+	}
+
+	aux = getenv("XDG_CONFIG_DIRS");
+	if (aux)
+		xdg_config_dirs = strdup(aux);
+	else
+		xdg_config_dirs = strdup(XDG_CONFIG_DIRS);
+
+	dirs = xdg_config_dirs;
+	/* split out xdg_config_dirs into tokens separated by ':' */
+	xdg_config_dir = calloc(xcd_size, sizeof(char *));
+	while ((aux = strsep(&xdg_config_dirs, ":")) != NULL) {
+		xdg_config_dir[i++] = strdup(aux);
+		if (i >= xcd_size) {
+			xcd_size *= 2;
+			xdg_config_dir = realloc(xdg_config_dir, xcd_size);
 		}
 	}
-	return GN_ERR_NONE;
+	free(dirs);
+
+	/* 1. $XDG_CONFIG_HOME/gnokii/config ($HOME/.config) */
+	snprintf(path, MAX_PATH_LEN, "%s/gnokii/config", xdg_config_home);
+	config_file_locations[(*retval)++] = strdup(path);
+
+	/* 2. $XDG_CONFIG_DIRS/gnokii/config (/etc/xdg) */
+	for (j = 0; j < i; j++) {
+		snprintf(path, MAX_PATH_LEN, "%s/gnokii/config", xdg_config_dir[j]);
+		config_file_locations[(*retval)++] = strdup(path);
+		CHECK_SIZE();
+		free(xdg_config_dir[j]);
+	}
+	free(xdg_config_dir);
+
+	/* old gnokii behaviour */
+
+	/* 3. $HOME/.gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "%s/.gnokiirc", home);
+	config_file_locations[(*retval)++] = strdup(path);
+	CHECK_SIZE();
+
+	/* 4. /etc/gnokiirc */
+	snprintf(path, MAX_PATH_LEN, "/etc/gnokiirc");
+	config_file_locations[(*retval)++] = strdup(path);
+
+	if (free_xdg_config_home)
+		free(xdg_config_home);
+
+	return config_file_locations;
+}
+#  endif /* !__MACH__ */
+#endif /* !WIN32 */
+
+GNOKII_API gn_error gn_cfg_read_default()
+{
+	gn_error error = GN_ERR_FAILED;
+	char **config_file_locations = NULL;
+	int num, i;
+
+	config_file_locations = get_locations(&num);
+
+	for (i = 0; i < num; i++) {
+		if (error != GN_ERR_NONE)
+			error = gn_cfg_file_read(config_file_locations[i]);
+		if (error != GN_ERR_NONE)
+			fprintf(stderr, _("Couldn't read %s config file.\n"), config_file_locations[i]);
+		free(config_file_locations[i]);
+	}		
+	free(config_file_locations);
+	return error;
 }
 
 /* DEPRECATED */
-GNOKII_API gn_error gn_cfg_read(char **bindir)
-{
-	gn_error retval;
-
-	retval = gn_cfg_read_default();
-
-	*bindir = gn_cfg_get(gn_cfg_info, "global", "bindir");
-	if (!*bindir)
-		*bindir = gn_cfg_get(gn_cfg_info, "gnokiid", "bindir");
-	if (!*bindir)
-		*bindir = "/usr/local/sbin";
-
-	return retval;
-}
-
 static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 {
 	char *val;
+	gn_error error;
+
+	error = gn_lib_init();
+	if (error != GN_ERR_NONE) {
+		fprintf(stderr, _("Failed to initialize libgnokii.\n"));
+		return error;
+	}
 
 	if (file == NULL && lines == NULL) {
-		dprintf("Couldn't open a config file or memory.\n");
+		fprintf(stderr, _("Couldn't open a config file or memory.\n"));
 		return GN_ERR_NOCONFIG;
 	}
 
@@ -637,14 +1088,10 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 
 	if (gn_cfg_info == NULL) {
 		/* this is bad, but the previous was much worse - bozo */
-		if (file)
-			dprintf("Couldn't open %s config file.\n", file);
-		else
-			dprintf("Couldn't read config\n");
 		return GN_ERR_NOCONFIG;
 	}
-	strcpy(gn_config_default.model, "");
-	strcpy(gn_config_default.port_device, "");
+	gn_config_default.model[0] = 0;
+	gn_config_default.port_device[0] = 0;
 	gn_config_default.connection_type = GN_CT_Serial;
 	gn_config_default.init_length = 0;
 	gn_config_default.serial_baudrate = 19200;
@@ -652,17 +1099,15 @@ static gn_error cfg_file_or_memory_read(const char *file, const char **lines)
 	gn_config_default.hardware_handshake = false;
 	gn_config_default.require_dcd = false;
 	gn_config_default.smsc_timeout = -1;
-	strcpy(gn_config_default.irda_string, "");
-	strcpy(gn_config_default.connect_script, "");
-	strcpy(gn_config_default.disconnect_script, "");
-	gn_config_default.rfcomm_cn = 1;
+	gn_config_default.irda_string[0] = 0;
+	gn_config_default.connect_script[0] = 0;
+	gn_config_default.disconnect_script[0] = 0;
+	gn_config_default.rfcomm_cn = 0;
 	gn_config_default.sm_retry = 0;
 	gn_config_default.use_locking = 0;
 
-	if (!cfg_psection_load(&gn_config_global, "global", &gn_config_default)) {
-		fprintf(stderr, _("No [global] section in %s config file.\n"), file);
-		return GN_ERR_NOPHONE;
-	}
+	if ((error = cfg_psection_load(&gn_config_global, "global", &gn_config_default)) != GN_ERR_NONE)
+		return error;
 
 	/* hack to support [sms] / smsc_timeout parameter */
 	if (gn_config_global.smsc_timeout < 0) {
@@ -698,13 +1143,14 @@ GNOKII_API gn_error gn_cfg_memory_read(const char **lines)
 GNOKII_API gn_error gn_cfg_phone_load(const char *iname, struct gn_statemachine *state)
 {
 	char section[256];
+	gn_error error;
 
 	if (iname == NULL || *iname == '\0') {
 		state->config = gn_config_global;
 	} else {
 		snprintf(section, sizeof(section), "phone_%s", iname);
-		if (!cfg_psection_load(&state->config, section, &gn_config_global))
-			return GN_ERR_NOPHONE;
+		if ((error = cfg_psection_load(&state->config, section, &gn_config_global)) != GN_ERR_NONE)
+			return error;
 	}
 
 	if (state->config.model[0] == '\0') {
