@@ -256,10 +256,12 @@ static const char *get_langinfo_codeset(void)
  */
 GNOKII_API const char *gn_char_get_encoding()
 {
-	if (*application_encoding)
-		return application_encoding; /* app has overriden encoding setting */
+	const char *coding;
+	if (*application_encoding) 
+		coding = application_encoding; /* app has overriden encoding setting */
 	else
-		return get_langinfo_codeset(); /* return default codeset */
+		coding = get_langinfo_codeset(); /* return default codeset */
+	return coding;
 }
 
 /**
@@ -300,8 +302,8 @@ int char_mblen(const char *src)
  * or -1 in case of error
  *
  * Converts a multibyte string to a wide char string.
- * Uses iconv() if available, else mbrtowc() if
- * available, else mbtowc().
+ * Uses iconv() if it is available and iconv_open() succeeds, else mbrtowc()
+ * if available, else mbtowc().
  */
 static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
 {
@@ -315,20 +317,27 @@ static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
 
 	pin = (char *)src;
 	pout = (char *)dst;
-	inlen = 4;
-	outlen = sizeof(wchar_t);
+	/* Let's assume that we have at most 4-bytes wide characters */
+	inlen = maxlen;
+	outlen = maxlen * sizeof(wchar_t);
 
 	cd = iconv_open("WCHAR_T", gn_char_get_encoding());
+	if (cd == (iconv_t)-1)
+		goto fallback;
 	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
+	if ((nconv == (size_t)-1) && (pin == src))
+		perror("char_mbtowc/iconv");
 	iconv_close(cd);
 
 	return (char*)dst == pout ? -1 : pin-src;
-#else
-#  ifdef HAVE_WCRTOMB
+fallback:
+#endif
+	if (maxlen >= MB_CUR_MAX)
+		maxlen = MB_CUR_MAX - 1;
+#ifdef HAVE_WCRTOMB
 	return mbrtowc(dst, src, maxlen, mbs);
-#  else
+#else
 	return mbtowc(dst, src, maxlen);
-#  endif
 #endif
 }
 
@@ -343,8 +352,8 @@ static int char_mbtowc(wchar_t *dst, const char *src, int maxlen, MBSTATE *mbs)
  * or -1 in case of error
  *
  * Converts a wide char string to a multibyte string.
- * Uses iconv() if available, else wcrtomb() if
- * available, else wctomb().
+ * Uses iconv() if it is available and iconv_open() succeeds, else wcrtomb()
+ * if available, else wctomb().
  */
 static int char_wctomb(char *dst, wchar_t src, MBSTATE *mbs)
 {
@@ -362,18 +371,20 @@ static int char_wctomb(char *dst, wchar_t src, MBSTATE *mbs)
 	outlen = 4;
 
 	cd = iconv_open(gn_char_get_encoding(), "WCHAR_T");
+	if (cd == (iconv_t)-1)
+		goto fallback;
 	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
 	if (nconv == (size_t)-1)
-		perror("iconv");
+		perror("char_wctomb/iconv");
 	iconv_close(cd);
 
 	return nconv == -1 ? -1 : pout-dst;
-#else
-    #ifdef HAVE_WCRTOMB
+fallback:
+#endif
+#ifdef HAVE_WCRTOMB
 	return wcrtomb(dst, src, mbs);
-    #else
+#else
 	return wctomb(dst, src);
-    #endif
 #endif
 }
 
@@ -485,8 +496,8 @@ GNOKII_API int gn_char_def_alphabet(unsigned char *string)
 		 *  - check in the default alphabet table
 		 *  - check in the extended default alphabet table
 		 */
-		if (!char_def_alphabet_ext(a + 256 * b) &&
-		    !char_def_alphabet(a + 256 * b)) {
+		if (!char_def_alphabet(256 * a + b) &&
+		    !char_def_alphabet_ext(256 * a + b)) {
 			free(ucs2str);
 			return false;
 		}
@@ -633,7 +644,7 @@ int char_7bit_pack(unsigned int offset, unsigned char *input,
 		bool double_char = false;
 		unsigned int a = 0xff & ucs2str[2 * i], b = 0xff & ucs2str[2 * i + 1];
 
-		in_num = a + 256 * b;
+		in_num = 256 * a + b;
 
 		if (char_def_alphabet_ext(in_num)) {
 			byte = GN_CHAR_UNI_ESCAPE;
@@ -795,8 +806,6 @@ size_t char_uni_alphabet_encode(const char *value, size_t n, wchar_t *dest, MBST
 {
 	int length;
 
-	if (n > MB_CUR_MAX)
-		n = MB_CUR_MAX;
 	length = char_mbtowc(dest, value, n, mbs);
 	return length;
 }
@@ -941,11 +950,17 @@ unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src,
  */
 unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src, int len)
 {
-	int length, offset = 0, pos = 0;
-	wchar_t  wc;
+	int pos = 0;
 	MBSTATE mbs;
+#ifndef HAVE_ICONV
+	int length, offset = 0;
+	wchar_t  wc;
+#endif
 
 	MBSTATE_ENC_CLEAR(mbs);
+#ifdef HAVE_ICONV
+	pos = ucs2_encode(dest, 2 * len, src, len);
+#else
 	while (offset < len) {
 		length = char_uni_alphabet_encode(src + offset, len - offset, &wc, &mbs);
 		switch (length) {
@@ -961,6 +976,7 @@ unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src,
 			break;
 		}
 	}
+#endif
 	return pos;
 }
 
@@ -1157,7 +1173,11 @@ int utf8_decode(char *outstring, size_t outlen, const char *instring, size_t inl
 	pout = outstring;
 
 	cd = iconv_open(gn_char_get_encoding(), "UTF-8");
+	if (cd == (iconv_t)-1)
+		return -1;
 	nconv = iconv(cd, &pin, &inlen, &pout, &outlen);
+	if (nconv == (size_t)-1)
+		perror("utf8_decode/iconv");
 	iconv_close(cd);
 	*pout = 0;
 #else
@@ -1225,8 +1245,12 @@ int utf8_encode(char *outstring, int outlen, const char *instring, int inlen)
 	pout = outstring;
 
 	cd = iconv_open("UTF-8", gn_char_get_encoding());
+	if (cd == (iconv_t)-1)
+		return -1;
 
 	nconv = iconv(cd, &pin, &inleft, &pout, &outleft);
+	if (nconv == (size_t)-1)
+		perror("utf8_encode/iconv");
 	*pout = 0;
 	iconv_close(cd);
 #else
@@ -1279,15 +1303,18 @@ int ucs2_encode(char *outstring, int outlen, const char *instring, int inlen)
 	pin = (char *)instring;
 	pout = outstring;
 
-	cd = iconv_open("UCS-2", gn_char_get_encoding());
+	cd = iconv_open("UCS-2BE", gn_char_get_encoding());
+	if (cd == (iconv_t)-1)
+		return -1;
 
 	nconv = iconv(cd, &pin, &inleft, &pout, &outleft);
-	*pout = 0;
+	if (nconv == (size_t)-1)
+		perror("ucs2_encode/iconv");
 	iconv_close(cd);
 	return (nconv < 0) ?  -1 : (char *)pout - outstring;
 #else
 	size_t nconv = char_unicode_encode(outstring, instring, inlen);
-	unsigned char *pin, *pout;
+
 	return (nconv < 0) ?  -1 : nconv;
 #endif
 }
