@@ -1,6 +1,6 @@
 /*
 
-  $Id: gsm-filetypes.c,v 1.74 2007/06/12 16:00:32 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -169,16 +169,17 @@ gn_error file_rtttl_load(FILE *file, gn_ringtone *ringtone)
 	unsigned char buffer[2000];
 	unsigned char *def, *notes, *ptr;
 
-	fread(buffer, 2000, 1, file);
+	if (fread(buffer, 1, 2000, file) < 1)
+		return GN_ERR_FAILED;
 
 	/* This is for buggy RTTTL ringtones without name. */
 	if (buffer[0] != RTTTL_SEP[0]) {
 		strtok(buffer, RTTTL_SEP);
-		sprintf(ringtone->name, "%s", buffer);
+		snprintf(ringtone->name, sizeof(ringtone->name), "%s", buffer);
 		def = strtok(NULL, RTTTL_SEP);
 		notes = strtok(NULL, RTTTL_SEP);
 	} else {
-		sprintf(ringtone->name, "GNOKII");
+		snprintf(ringtone->name, sizeof(ringtone->name), "GNOKII");
 		def = strtok(buffer, RTTTL_SEP);
 		notes = strtok(NULL, RTTTL_SEP);
 	}
@@ -281,7 +282,7 @@ gn_error file_nokraw_load(FILE *file, gn_ringtone *ringtone)
 	int n;
 	gn_error err;
 
-	strcpy(ringtone->name, "GNOKII");
+	snprintf(ringtone->name, sizeof(ringtone->name), "GNOKII");
 
 	if ((n = fread(buf, 1, sizeof(buf), file)) < 0) return GN_ERR_UNKNOWN;
 
@@ -539,8 +540,9 @@ gn_error file_nokraw_save(FILE *file, gn_ringtone *ringtone, int dct4)
 gn_error gn_file_bitmap_read(char *filename, gn_bmp *bitmap, gn_phone *info)
 {
 	FILE *file;
-	unsigned char buffer[300];
+	unsigned char buffer[9];
 	int error;
+	size_t count;
 	gn_filetypes filetype = GN_FT_None;
 
 	file = fopen(filename, "rb");
@@ -548,21 +550,21 @@ gn_error gn_file_bitmap_read(char *filename, gn_bmp *bitmap, gn_phone *info)
 	if (!file)
 		return GN_ERR_FAILED;
 
-	fread(buffer, 1, 9, file); /* Read the header of the file. */
+	count = fread(buffer, 1, 9, file); /* Read the header of the file. */
 
 	/* Attempt to identify filetype */
 
-	if (memcmp(buffer, "NOL", 3) == 0) {               /* NOL files have 'NOL' at the start */
+	if (count >= 3 && memcmp(buffer, "NOL", 3) == 0) {               /* NOL files have 'NOL' at the start */
 		filetype = GN_FT_NOL;
-	} else if (memcmp(buffer, "NGG", 3) == 0) {        /* NGG files have 'NGG' at the start */
+	} else if (count >= 3 && memcmp(buffer, "NGG", 3) == 0) {        /* NGG files have 'NGG' at the start */
 		filetype = GN_FT_NGG;
-	} else if (memcmp(buffer, "FORM", 4) == 0) {       /* NSL files have 'FORM' at the start */
+	} else if (count >= 4 && memcmp(buffer, "FORM", 4) == 0) {       /* NSL files have 'FORM' at the start */
 		filetype = GN_FT_NSL;
-	} else if (memcmp(buffer, "NLM", 3) == 0) {        /* NLM files have 'NLM' at the start */
+	} else if (count >= 3 && memcmp(buffer, "NLM", 3) == 0) {        /* NLM files have 'NLM' at the start */
 		filetype = GN_FT_NLM;
-	} else if (memcmp(buffer, "BM", 2) == 0) {         /* BMP, I61 and GGP files have 'BM' at the start */
+	} else if (count >= 2 && memcmp(buffer, "BM", 2) == 0) {         /* BMP, I61 and GGP files have 'BM' at the start */
 		filetype = GN_FT_BMP;
-	} else if (memcmp(buffer, "/* XPM */", 9) == 0) {  /* XPM files have 'XPM' at the start */
+	} else if (count >= 9 && memcmp(buffer, "/* XPM */", 9) == 0) {  /* XPM files have 'XPM' at the start */
 		filetype = GN_FT_XPMF;
 	}
 
@@ -665,7 +667,9 @@ gn_error file_bmp_load(FILE *file, gn_bmp *bitmap)
 
 	gn_bmp_clear(bitmap);
 
-	fread(buffer, 1, 34, file); /* required part of header */
+	/* required part of header */
+	if (fread(buffer, 1, 34, file) != 34)
+		return GN_ERR_FAILED;
 
 	h = buffer[22] + 256 * buffer[21]; /* height of image in the file */
 	w = buffer[18] + 256 * buffer[17]; /* width of image in the file */
@@ -710,7 +714,9 @@ gn_error file_bmp_load(FILE *file, gn_bmp *bitmap)
 	}
 
 	pos = buffer[10] - 34;
-	fread(buffer, 1, pos, file); /* the rest of the header (if exists) and the color palette */
+	/* the rest of the header (if exists) and the color palette */
+	if (fread(buffer, 1, pos, file) != pos)
+		return GN_ERR_WRONGDATAFORMAT;
 
 	dprintf("First color in BMP file: %i %i %i ", buffer[pos-8], buffer[pos-7], buffer[pos-6]);
 	if (buffer[pos-8] == 0x00 && buffer[pos-7] == 0x00 && buffer[pos-6] == 0x00) dprintf("(black)");
@@ -733,10 +739,11 @@ gn_error file_bmp_load(FILE *file, gn_bmp *bitmap)
 		i = 1;
 		for (x = 0; x < w; x++) {
 			if (pos == 7) { /* new byte ! */
-				fread(buffer, 1, 1, file);
+				if (fread(buffer, 1, 1, file) != 1)
+					return GN_ERR_WRONGDATAFORMAT;
 				sizeimage++;
 				i++;
-				if (i == 5) i = 1; /* each line is written in multiply of 4 bytes */
+				if (i == 5) i = 1; /* each line is written in multiples of 4 bytes */
 			}
 			if (x <= bitmap->width && y <= bitmap->height) { /* we have top left corner ! */
 				if (first_black) {
@@ -751,7 +758,8 @@ gn_error file_bmp_load(FILE *file, gn_bmp *bitmap)
 		pos = 7; /* going to the new byte */
 		if (i != 1) {
 			while (i != 5) { /* each line is written in multiples of 4 bytes */
-				fread(buffer, 1, 1, file);
+				if (fread(buffer, 1, 1, file) != 1)
+					return GN_ERR_WRONGDATAFORMAT;
 				sizeimage++;
 				i++;
 			}
@@ -766,9 +774,9 @@ gn_error file_nol_load(FILE *file, gn_bmp *bitmap, gn_phone *info)
 	unsigned char buffer[GN_BMP_MAX_SIZE + 20];
 	int i, j;
 
-
-	fread(buffer, 1, 20, file);
-	sprintf(bitmap->netcode, "%d %02d", buffer[6] + 256 * buffer[7], buffer[8]);
+	if (fread(buffer, 1, 20, file) != 20)
+		return GN_ERR_FAILED;
+	snprintf(bitmap->netcode, sizeof(bitmap->netcode), "%d %02d", buffer[6] + 256 * buffer[7], buffer[8]);
 
 	bitmap->width = buffer[10];
 	bitmap->height = buffer[12];
@@ -812,7 +820,8 @@ gn_error file_ngg_load(FILE *file, gn_bmp *bitmap, gn_phone *info)
 
 	bitmap->type = GN_BMP_CallerLogo;
 
-	fread(buffer, 1, 16, file);
+	if (fread(buffer, 1, 16, file) != 16)
+		return GN_ERR_FAILED;
 	bitmap->width = buffer[6];
 	bitmap->height = buffer[8];
 	bitmap->size = bitmap->height * bitmap->width / 8;
@@ -906,8 +915,10 @@ gn_error file_nlm_load(FILE *file, gn_bmp *bitmap)
 	int pos, pos2, x, y;
 	div_t division;
 
-	fread(buffer, 1, 5, file);
-	fread(buffer, 1, 1, file);
+	if (fread(buffer, 1, 5, file) != 5)
+		return GN_ERR_FAILED;
+	if (fread(buffer, 1, 1, file) != 1)
+		return GN_ERR_FAILED;
 
 	switch (buffer[0]) {
 	case 0x00:
@@ -926,7 +937,8 @@ gn_error file_nlm_load(FILE *file, gn_bmp *bitmap)
 		return GN_ERR_WRONGDATAFORMAT;
 	}
 
-	fread(buffer, 1, 4, file);
+	if (fread(buffer, 1, 4, file) != 4)
+		return GN_ERR_FAILED;
 	bitmap->width = buffer[1];
 	bitmap->height = buffer[2];
 	bitmap->size = bitmap->width * bitmap->height / 8;
@@ -956,7 +968,8 @@ gn_error file_ota_load(FILE *file, gn_bmp *bitmap, gn_phone *info)
 	char buffer[4];
 
 	/* We could check for extended info here - indicated by the 7th bit being set in the first byte */
-	fread(buffer, 1, 4, file);
+	if (fread(buffer, 1, 4, file) != 4)
+		return GN_ERR_FAILED;
 
 	bitmap->width = buffer[1];
 	bitmap->height = buffer[2];
@@ -1052,44 +1065,6 @@ gn_error gn_file_bitmap_save(char *filename, gn_bmp *bitmap, gn_phone *info)
 	return GN_ERR_NONE;
 }
 
-
-/* FIXME - this should not ask for confirmation here - I'm not sure what calls it though */
-/* mode == 0 -> overwrite
- * mode == 1 -> ask
- * mode == 2 -> append
- */
-int gn_file_text_save(char *filename, char *text, int mode)
-{
-	FILE *file;
-	int confirm = -1;
-	char ans[5];
-	struct stat buf;
-
-	/* Ask before overwriting */
-	if ((mode == 1) && (stat(filename, &buf) == 0)) {
-		fprintf(stdout, _("File %s exists.\n"), filename);
-		while (confirm < 0) {
-			fprintf(stdout, _("Overwrite? (yes/no) "));
-			gn_line_get(stdin, ans, 4);
-			if (!strcmp(ans, _("yes"))) confirm = 1;
-			else if (!strcmp(ans, _("no"))) confirm = 0;
-		}
-		if (!confirm) return -1;
-	}
-
-	if (mode == 2) file = fopen(filename, "a");
-	else file = fopen(filename, "w");
-
-	if (!file) {
-		fprintf(stderr, _("Can't open file %s for writing!\n"),  filename);
-		return -1;
-	}
-	fprintf(file, "%s\n", text);
-	fclose(file);
-	return 2;
-}
-
-
 #ifdef XPM
 void file_xpm_save(char *filename, gn_bmp *bitmap)
 {
@@ -1164,13 +1139,13 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 			if (pos == 7) { //new byte !
 				sizeimage++;
 				i++;
-				if (i == 5) i = 1; //each line is written in multiply of 4 bytes
+				if (i == 5) i = 1; //each line is written in multiples of 4 bytes
 			}
 			pos--;
 			if (pos < 0) pos = 7; //going to new byte
 		}
 		pos = 7; //going to new byte
-		while (i != 5) { //each line is written in multiply of 4 bytes
+		while (i != 5) { //each line is written in multiples of 4 bytes
 			sizeimage++;
 			i++;
 		}
@@ -1195,7 +1170,7 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 			if (pos == 7) { //new byte !
 				if (x != 0) fwrite(buffer, 1, sizeof(buffer), file);
 				i++;
-				if(i == 5) i = 1; //each line is written in multiply of 4 bytes
+				if(i == 5) i = 1; //each line is written in multiples of 4 bytes
 				buffer[0] = 0;
 			}
 			if (gn_bmp_point(bitmap, x, y)) buffer[0] |= (1 << pos);
@@ -1204,7 +1179,7 @@ void file_bmp_save(FILE *file, gn_bmp *bitmap)
 		}
 		pos = 7; //going to new byte
 		fwrite(buffer, 1, sizeof(buffer), file);
-		while (i != 5) { //each line is written in multiply of 4 bytes
+		while (i != 5) { //each line is written in multiples of 4 bytes
 			buffer[0] = 0;
 			fwrite(buffer, 1, sizeof(buffer), file);
 			i++;
@@ -1445,7 +1420,6 @@ inline int local_atoi(char *str, int len)
 
 GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char *line)
 {
-	char backline[MAX_INPUT_LINE_LEN];
 	char memory_type_char[3];
 	char number[GN_PHONEBOOK_NUMBER_MAX_LENGTH];
 	int length, o, offset = 0;
@@ -1454,7 +1428,6 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 	memset(entry, 0, sizeof(gn_phonebook_entry));
 
 	length = strlen(line);
-	strcpy(backline, line);
 	entry->empty = true;
 	memory_type_char[2] = 0;
 
@@ -1542,6 +1515,11 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 	entry->empty = false;
 
 	for (entry->subentries_count = 0; offset < length; entry->subentries_count++) {
+		if (entry->subentries_count == GN_PHONEBOOK_SUBENTRIES_MAX_NUMBER) {
+			fprintf(stderr, _("Formatting error: too many subentries\n"));
+			error = GN_ERR_WRONGDATAFORMAT;
+			goto endloop;
+		}
 		memset(number, 0, sizeof(number));
 		GET_NEXT_TOKEN();
 		STORE_TOKEN(number);
@@ -1627,6 +1605,9 @@ GNOKII_API gn_error gn_file_phonebook_raw_parse(gn_phonebook_entry *entry, char
 				entry->subentries[entry->subentries_count].data.date.minute = local_atoi(line + offset + 10, 2);
 				entry->subentries[entry->subentries_count].data.date.second = local_atoi(line + offset + 12, 2);
 				break;
+			case GN_PHONEBOOK_ENTRY_ExtGroup:
+				entry->subentries[entry->subentries_count].data.id = local_atoi(line + offset, 3);
+				break;
 			default:
 				STORE_TOKEN(entry->subentries[entry->subentries_count].data.number);
 				break;
@@ -1651,7 +1632,8 @@ endloop:
 		entry->subentries[entry->subentries_count].entry_type   = GN_PHONEBOOK_ENTRY_Number;
 		entry->subentries[entry->subentries_count].number_type  = GN_PHONEBOOK_NUMBER_General;
 		entry->subentries[entry->subentries_count].id = 2;
-		strcpy(entry->subentries[entry->subentries_count].data.number, entry->number);
+		snprintf(entry->subentries[entry->subentries_count].data.number,
+			sizeof(entry->subentries[entry->subentries_count].data.number), "%s", entry->number);
 		entry->subentries_count = 1;
 	}
 	return error;
@@ -1709,6 +1691,13 @@ GNOKII_API gn_error gn_file_phonebook_raw_write(FILE *f, gn_phonebook_entry *ent
 				entry->subentries[i].data.date.year, entry->subentries[i].data.date.month, entry->subentries[i].data.date.day,
 				entry->subentries[i].data.date.hour, entry->subentries[i].data.date.minute, entry->subentries[i].data.date.second);
 			break;
+		case GN_PHONEBOOK_ENTRY_ExtGroup:
+			fprintf(f, ";%d;%d;%d;%03d",
+				entry->subentries[i].entry_type,
+				entry->subentries[i].number_type,
+				entry->subentries[i].id,
+				entry->subentries[i].data.id);
+			break;
 		default:
 			add_slashes(escaped_name, entry->subentries[i].data.number, sizeof(escaped_name), strlen(entry->subentries[i].data.number));
 			fprintf(f, ";%d;%d;%d;%s",
