@@ -93,7 +93,7 @@ static gn_phone_model models[] = {
 	{"1100",  "RH-18",	PM_OLD_DEFAULT },
 	{"1100",  "RH-38",	PM_OLD_DEFAULT },
 	{"1100b", "RH-36",	PM_OLD_DEFAULT },
-	{"1209",  "RH-105",	PM_OLD_DEFAULT },	/* phone supports a very limited command set */
+	{"1209",  "RH-105",	PM_OLD_DEFAULT }, /* Series 30, phone supports a very limited command set */
 	{"1220",  "NKC-1",	PM_OLD_DEFAULT },
 	{"1260",  "NKW-1",	PM_OLD_DEFAULT },
 	{"1261",  "NKW-1C",	PM_OLD_DEFAULT },
@@ -129,13 +129,14 @@ static gn_phone_model models[] = {
 	{"2600",  "RH-59",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"2630",  "RM-298",	PM_DEFAULT },
 	{"2650",  "RH-53",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
+	{"2760",  "RM-258",	PM_DEFAULT }, /* Series 40 5th Edition */
 	{"3100",  "RH-19",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"3100b", "RH-50",	PM_DEFAULT },
 	{"3105",  "RH-48",	PM_OLD_DEFAULT },
 	{"3108",  "RH-6",	PM_OLD_DEFAULT },
 	{"3110",  "NHE-8",	PM_OLD_DEFAULT | PM_DATA },
 	{"3110",  "0310" ,	PM_OLD_DEFAULT | PM_DATA }, /* NHE-8 */
-	{"3110c", "RM-237",     PM_DEFAULT | PM_SMSFILE }, /* Nokia 3110 classic */
+	{"3110c", "RM-237",     PM_DEFAULT | PM_SMSFILE | PM_EXTPBK2 | PM_EXTCALENDAR }, /* Nokia 3110 classic */
 	{"3120",  "RH-19",	PM_DEFAULT },
 	{"3120b", "RH-50",	PM_DEFAULT },
 	{"3200",  "RH-30",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
@@ -188,6 +189,7 @@ static gn_phone_model models[] = {
 	{"5120",  "NSC-1",	PM_OLD_DEFAULT  },
 	{"5125",  "NSC-1",	PM_OLD_DEFAULT },
 	{"5130",  "NSK-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
+	{"5130 XpressMusic",  "RM-495",	PM_DEFAULT },
 	{"5140",  "NPL-5",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"5140i", "RM-104",	PM_DEFAULT },
 	{"5160",  "NSW-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
@@ -196,7 +198,8 @@ static gn_phone_model models[] = {
 	{"5185",  "NSD-1A",	PM_OLD_DEFAULT },
 	{"5190",  "NSB-1",	PM_OLD_DEFAULT | PM_NETMONITOR | PM_DATA | PM_AUTHENTICATION },
 	{"5210",  "NSM-5",	PM_DEFAULT },
-	{"5300",  "RM-146",	PM_DEFAULT },
+	{"5300 XpressMusic",  "RM-146",	PM_DEFAULT }, /* Series 40 3rd Edition */
+	{"5310 XpressMusic",  "RM-303",	PM_DEFAULT },
 	{"5510",  "NPM-5",	PM_OLD_DEFAULT },
 	{"6010",  "NPM-10",	PM_OLD_DEFAULT },
 	{"6010",  "NPM-10X",	PM_OLD_DEFAULT },
@@ -210,6 +213,7 @@ static gn_phone_model models[] = {
 	{"6080",  "NME-2",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2A",	PM_OLD_DEFAULT },
 	{"6081",  "NME-2E",	PM_OLD_DEFAULT },
+	{"6085",  "RM-260",     PM_DEFAULT },
 	{"6090",  "NME-3",	PM_OLD_DEFAULT },
 	{"6100",  "NPL-2",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6108",  "RH-4",	PM_OLD_DEFAULT },
@@ -238,7 +242,8 @@ static gn_phone_model models[] = {
 	{"6233",  "RM123_145_-", PM_DEFAULT | PM_NETMONITOR },
 	{"6250",  "NHM-3",	PM_DEFAULT | PM_NETMONITOR },
 	{"6280",  "RH-78",	PM_DEFAULT | PM_NETMONITOR },
-	{"6300",  "RM-217",     PM_DEFAULT | PM_FULLPBK | PM_SMSFILE },
+	{"6300",  "RM-217",     PM_DEFAULT | PM_FULLPBK | PM_SMSFILE | PM_EXTPBK2 | PM_EXTCALENDAR },
+	{"6303",  "RM-443",     PM_DEFAULT | PM_FULLPBK | PM_SMSFILE | PM_EXTPBK2 | PM_EXTCALENDAR }, /* Nokia 6303 classic */
 	{"6310",  "NPE-4",	PM_DEFAULT },
 	{"6310i", "NPL-1",	PM_DEFAULT },
 	{"6340",  "NPM-2",	PM_OLD_DEFAULT },
@@ -257,6 +262,7 @@ static gn_phone_model models[] = {
 	{"6610i", "RM-37",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6650",  "NHM-1",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6680",  "RM-36",	PM_DEFAULT | PM_NETMONITOR },
+	{"6700 classic",  "RM-470",	PM_DEFAULT },  /* Series 40 6th Edition */
 	{"6800",  "NHL-6",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6800",  "NSB-9",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
 	{"6810",  "RM-2",	PM_DEFAULT | PM_XGNOKIIBREAKAGE },
@@ -321,7 +327,7 @@ static gn_phone_model models[] = {
 	{"N-Gage", "NEM-4",	PM_OLD_DEFAULT },
 	{"RinGo", "NHX-7",	PM_OLD_DEFAULT },
 	{"sx1",  "SX1",		PM_DEFAULT },
-	{"SIM/USIM", "pcsc",	0 },
+	{"SIM/USIM", "APDU",	0 },
 	{NULL,    NULL, 0 } /* keep last one as NULL */
 };
 
