@@ -1,6 +1,6 @@
 /*
 
-  $Id: nokia-decoding.c,v 1.33 2007/05/04 19:45:21 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -36,98 +36,14 @@
 
 */
 
-#include <time.h>
 #include "config.h"
+
+#include <time.h>
 #include "gnokii-internal.h" 
 #include "gnokii.h"
 #include "nokia-decoding.h"
 #include "compat.h"
 
-GNOKII_API char *gn_subentrytype2string(gn_phonebook_entry_type entry_type, gn_phonebook_number_type number_type)
-{
-	switch (entry_type) {
-	case GN_PHONEBOOK_ENTRY_Name:
-		return _("Name");
-	case GN_PHONEBOOK_ENTRY_Email:
-		return _("Email");
-	case GN_PHONEBOOK_ENTRY_Postal:
-		return _("Postal address");
-	case GN_PHONEBOOK_ENTRY_Note:
-	 	return _("Note");
-	case GN_PHONEBOOK_ENTRY_Number:
-		switch (number_type) {
-		case GN_PHONEBOOK_NUMBER_Home:
-			return _("Home number");
-		case GN_PHONEBOOK_NUMBER_Mobile:
-			return _("Mobile number");
-		case GN_PHONEBOOK_NUMBER_Fax:
-			return _("Fax number");
-		case GN_PHONEBOOK_NUMBER_Work:
-			return _("Business number");
-		case GN_PHONEBOOK_NUMBER_None:
-		case GN_PHONEBOOK_NUMBER_Common:
-		case GN_PHONEBOOK_NUMBER_General:
-			return _("General number");
-		default:
-			return _("Unknown number");
-		}
-	case GN_PHONEBOOK_ENTRY_Ringtone:
-		return _("Ringtone");
-	case GN_PHONEBOOK_ENTRY_Date:
-		return _("Date");
-	case GN_PHONEBOOK_ENTRY_Pointer:
-		return _("Pointer");
-	case GN_PHONEBOOK_ENTRY_Logo:
-		return _("Logo");
-	case GN_PHONEBOOK_ENTRY_LogoSwitch:
-		return _("Logo switch");
-	case GN_PHONEBOOK_ENTRY_Group:
-		return _("Group");
-	case GN_PHONEBOOK_ENTRY_URL:
-		return _("URL");
-	case GN_PHONEBOOK_ENTRY_Location:
-		return _("Location");
-	case GN_PHONEBOOK_ENTRY_Image:
-		return _("Image");
-	case GN_PHONEBOOK_ENTRY_RingtoneAdv:
-		return _("Ringtone");
-	case GN_PHONEBOOK_ENTRY_UserID:
-		return _("User ID");
-	case GN_PHONEBOOK_ENTRY_PTTAddress:
-		return _("Push-to-talk address");
-	case GN_PHONEBOOK_ENTRY_FirstName:
-		return _("First name");
-	case GN_PHONEBOOK_ENTRY_LastName:
-		return _("Last name");
-	case GN_PHONEBOOK_ENTRY_PostalAddress:
-		return _("Postal address");
-	case GN_PHONEBOOK_ENTRY_ExtendedAddress:
-		return _("Extended address");
-	case GN_PHONEBOOK_ENTRY_Street:
-		return _("Street");
-	case GN_PHONEBOOK_ENTRY_City:
-		return _("City");
-	case GN_PHONEBOOK_ENTRY_StateProvince:
-		return _("State or province");
-	case GN_PHONEBOOK_ENTRY_ZipCode:
-		return _("Zip code");
-	case GN_PHONEBOOK_ENTRY_Country:
-		return _("Country");
-	case GN_PHONEBOOK_ENTRY_FormalName:
-		return _("Formal name");
-	case GN_PHONEBOOK_ENTRY_JobTitle:
-		return _("Job title");
-	case GN_PHONEBOOK_ENTRY_Company:
-		return _("Company");
-	case GN_PHONEBOOK_ENTRY_Nickname:
-		return _("Nickname");
-	case GN_PHONEBOOK_ENTRY_Birthday:
-		return _("Birthday date");
-	default:
-		return _("Unknown");
-	}
-}
-
 gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 			  int blocks, int memtype, int speeddial_pos)
 {
@@ -294,7 +210,12 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 			subentry->entry_type  = blockstart[0];
 			subentry->number_type = 0;
 			subentry->id          = blockstart[4];
-			char_unicode_decode(subentry->data.number, (blockstart + 6), blockstart[5]);
+#ifdef DEBUG
+			if (blockstart[5] > GN_PHONEBOOK_NAME_MAX_LENGTH * 2) {
+				dprintf("Entry too long: truncated from %d to %d bytes\n", blockstart[5] / 2, GN_PHONEBOOK_NAME_MAX_LENGTH);
+			}
+#endif
+			char_unicode_decode(subentry->data.number, (blockstart + 6), GNOKII_MIN(blockstart[5], GN_PHONEBOOK_NAME_MAX_LENGTH * 2));
 			dprintf("   Type: %d (%02x)\n", subentry->entry_type, subentry->entry_type);
 			dprintf("   Text: %s\n", subentry->data.number);
 			subblock_count++;
@@ -310,7 +231,7 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 			subentry->id          = blockstart[4];
 			char_unicode_decode(subentry->data.number, (blockstart + 10), blockstart[9]);
 			if (!subblock_count)
-				strcpy(data->phonebook_entry->number, subentry->data.number);
+				snprintf(data->phonebook_entry->number, sizeof(data->phonebook_entry->number), "%s", subentry->data.number);
 			dprintf("   Type: %d (%02x)\n", subentry->number_type, subentry->number_type);
 			dprintf("   Number: %s\n", subentry->data.number);
 			subblock_count++;
@@ -369,6 +290,18 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 				data->bitmap->number = blockstart[5] - 1;
 			dprintf("   Group: %d\n", blockstart[5] - 1);
 			break;
+		case GN_PHONEBOOK_ENTRY_ExtGroup: /* Extended caller group number: in Series40 3rd Ed and later groups are defined by users */
+			if (!subentry) {
+				dprintf("ERROR!!!");
+				break;
+			}
+			subentry->entry_type  = blockstart[0];
+			subentry->number_type = 0;
+			subentry->id          = blockstart[4];
+			subentry->data.id     = blockstart[7];
+			subblock_count++;
+			data->phonebook_entry->subentries_count++;
+			break;
 		case GN_PHONEBOOK_ENTRY_RingtoneAdv:   /* Newer ringtones */
 			switch(blockstart[15]) {
 			case 0x01:
@@ -394,18 +327,35 @@ gn_error phonebook_decode(unsigned char *blockstart, int length, gn_data *data,
 			break;
 		default:
 			dprintf("Unknown phonebook block %02x\n", blockstart[0]);
+			{
+				int j;
+				for (j = 0; j < blockstart[3]; j++)
+					dprintf("%02x ", blockstart[j]);
+				dprintf("\n");
+			}
 			break;
 		}
 		blockstart += blockstart[3];
 	}
 	if (data->bitmap && data->bitmap->text[0] == '\0') {
 		switch (data->bitmap->number) {
-		case 0: strcpy(data->bitmap->text, _("Family")); break;
-		case 1: strcpy(data->bitmap->text, _("VIP")); break;
-		case 2: strcpy(data->bitmap->text, _("Friends")); break;
-		case 3: strcpy(data->bitmap->text, _("Colleagues")); break;
-		case 4: strcpy(data->bitmap->text, _("Other")); break;
-		default: break;
+		case 0:
+			snprintf(data->bitmap->text, sizeof(data->bitmap->text), "%s", _("Family"));
+			break;
+		case 1:
+			snprintf(data->bitmap->text, sizeof(data->bitmap->text), "%s", _("VIP"));
+			break;
+		case 2:
+			snprintf(data->bitmap->text, sizeof(data->bitmap->text), "%s", _("Friends"));
+			break;
+		case 3:
+			snprintf(data->bitmap->text, sizeof(data->bitmap->text), "%s", _("Colleagues"));
+			break;
+		case 4:
+			snprintf(data->bitmap->text, sizeof(data->bitmap->text), "%s", _("Other"));
+			break;
+		default:
+			break;
 		}
 	}
 	return GN_ERR_NONE;
