@@ -1,6 +1,6 @@
 /*
 
-  $Id: vcal.c,v 1.17 2007/04/29 10:37:27 dforsi Exp $
+  $Id$
 
   G N O K I I
 
@@ -40,11 +40,10 @@
  * Define it unconditionally to have nicer code below. */
 static const char gnokii_version[] = VERSION;
 
-
 #ifdef HAVE_LIBICAL
 /* needs gnokii-internal.h for string_base64() and utf8_encode() */
 #  include "gnokii-internal.h"
-#  include "ical.h"
+#  include <libical/ical.h>
 /* mother's little helpers */
 static int iterate_cal(icalcomponent *parent, int depth, int *count, icalcomponent **result, icalcomponent_kind kind);
 static const char *comp_get_category(icalcomponent *comp);
@@ -59,30 +58,66 @@ static char *ical_fgets(char *s, size_t size, void *d)
 
 /* This string is used so often that we keep it as long as the program is
  * running. Due to VERSION being defined in both libical and gnokii we need
- * to us a static string instead of a pre-compiler macro.
+ * to use a static string instead of a pre-processor macro.
  */
 static char vcal_prodid[64] = {0};
 static inline const char *get_prodid()
 {
 	if (!*vcal_prodid) {
-		sprintf(vcal_prodid, "//Gnokii.org//NONSGML v%s", gnokii_version);
+		snprintf(vcal_prodid, sizeof(vcal_prodid), "//Gnokii.org//NONSGML v%s", gnokii_version);
 	}
 	return vcal_prodid;
 }
 
+typedef int (*print_func) (void *data, const char *fmt, ...);
+
+typedef struct {
+	char *str;
+	char *end;
+	unsigned int len;
+} ical_string;
+
+static void ical_append_printf(ical_string *str, const char *fmt, ...)
+{
+	char buf[1024];
+	va_list ap;
+	int len;
+
+	va_start(ap, fmt);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
+	va_end(ap);
+
+	if (str->str == NULL) {
+		str->str = strdup (buf);
+		str->len = strlen (buf) + 1;
+		return;
+	}
+
+	len = strlen (buf);
+	str->str = realloc(str->str, len + str->len);
+
+	memcpy (str->str + str->len - 1, buf, len);
+	str->len = str->len + len;
+	str->end = str->str + str->len;
+
+	/* NULL-terminate the string now */
+	*(str->end - 1) = '\0';
+}
+
 /*
 	ICALENDAR Reading functions
  */
-GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
+GNOKII_API char *gn_calnote2icalstr(gn_calnote *calnote)
 {
+	ical_string str;
+
 #ifdef HAVE_LIBICAL
-#  define MAX_PROP_INDEX 5
-	icalcomponent *pIcal = NULL;
-	struct icaltimetype stime = {0}, etime = {0}, atime = {0};
-	icalproperty *properties[MAX_PROP_INDEX+1] = {0}; /* order and number of properties vary */
-	int iprop = 0;
+	icalcomponent *pIcal = NULL, *vevent;
+	struct icaltimetype stime = {0}, etime = {0};
 	char compuid[64];
 
+	memset(&str, 0, sizeof (str));
+
 	/* In at least some Nokia phones it is possible to skip the year of
 	   birth, in this case year == 0xffff, so we set it to the arbitrary
 	   value of 1800 */
@@ -96,6 +131,18 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 
 	stime.is_daylight = 1;
 
+	snprintf(compuid, sizeof(compuid), "guid.gnokii.org_%d_%d", calnote->location, rand());
+
+	vevent = icalcomponent_vanew(ICAL_VEVENT_COMPONENT,
+				     icalproperty_new_dtstart(stime),
+				     icalproperty_new_uid(compuid),
+				     icalproperty_new_categories("GNOKII"),
+				     0);
+	if (!vevent) {
+		dprintf("ERROR in icalcomponent_vanew() creating VEVENT\n");
+		return NULL;
+	}
+
 	if (calnote->end_time.year) {
 		etime.year = (calnote->end_time.year == 0xffff ? 1800 : calnote->end_time.year);
 		etime.month = calnote->end_time.month;
@@ -105,76 +152,106 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 		etime.second = calnote->end_time.second;
 		etime.is_daylight = 1;
 
-		properties[iprop++] = icalproperty_new_dtend(etime);
+		icalcomponent_add_property(vevent, icalproperty_new_dtend(etime));
 	}
 
-	/* FIXME: how to set alarm?
-	if (calnote->alarm.enabled) {
-		atime.year = (calnote->alarm.timestamp.year == 0xffff ? 1800 : calnote->alarm.timestamp.year);
-		atime.month = calnote->alarm.timestamp.month;
-		atime.day = calnote->alarm.timestamp.day;
-		atime.hour = calnote->alarm.timestamp.hour;
-		atime.minute = calnote->alarm.timestamp.minute;
-		atime.second = calnote->alarm.timestamp.second;
-		atime.is_daylight = 1;
-
-		properties[iprop++] = icalproperty_new_FIXME(atime);
-	}
-	*/
-
 	/* TODO: should the strings be configurable? */
 	switch(calnote->type) {
 	case GN_CALNOTE_MEMO:
-		properties[iprop++] = icalproperty_new_categories("MISCELLANEOUS");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("MISCELLANEOUS"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	case GN_CALNOTE_REMINDER:
-		properties[iprop++] = icalproperty_new_categories("REMINDER");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("REMINDER"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	case GN_CALNOTE_CALL:
-		properties[iprop++] = icalproperty_new_categories("PHONE CALL");
-		properties[iprop++] = icalproperty_new_summary(calnote->phone_number);
-		properties[iprop++] = icalproperty_new_description(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("PHONE CALL"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->phone_number));
+		icalcomponent_add_property(vevent, icalproperty_new_description(calnote->text));
 		break;
 	case GN_CALNOTE_MEETING:
-		properties[iprop++] = icalproperty_new_categories("MEETING");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("MEETING"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		if (calnote->mlocation[0])
-			properties[iprop++] = icalproperty_new_location(calnote->mlocation);
+			icalcomponent_add_property(vevent, icalproperty_new_location(calnote->mlocation));
 		break;
 	case GN_CALNOTE_BIRTHDAY:
-		properties[iprop++] = icalproperty_new_categories("ANNIVERSARY");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
-		do {
-			char rrule[64];
-			sprintf(rrule, "FREQ=YEARLY;INTERVAL=1;BYMONTH=%d", stime.month);
-			properties[iprop++] = icalproperty_new_rrule(icalrecurrencetype_from_string(rrule));
-		} while (0);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("ANNIVERSARY"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		stime.is_date = 1;
 		calnote->recurrence = GN_CALNOTE_YEARLY;
 		break;
 	default:
-		properties[iprop++] = icalproperty_new_categories("UNKNOWN");
-		properties[iprop++] = icalproperty_new_summary(calnote->text);
+		icalcomponent_add_property(vevent, icalproperty_new_categories("UNKNOWN"));
+		icalcomponent_add_property(vevent, icalproperty_new_summary(calnote->text));
 		break;
 	}
 
-	sprintf(compuid, "guid.gnokii.org_%d_%d", calnote->location, rand());
+	if (calnote->recurrence) {
+		char rrule[64];
+		char *freq;
+		int interval = 1;
+
+		switch (calnote->recurrence) {
+		case GN_CALNOTE_NEVER:
+			goto norecurrence;
+		case GN_CALNOTE_DAILY:
+			freq = "DAILY";
+			break;
+		case GN_CALNOTE_WEEKLY:
+			freq = "WEEKLY";
+			break;
+		case GN_CALNOTE_2WEEKLY:
+			freq = "WEEKLY";
+			interval = 2;
+			break;
+		case GN_CALNOTE_MONTHLY:
+			freq = "MONTHLY";
+			break;
+		case GN_CALNOTE_YEARLY:
+			freq = "YEARLY";
+			break;
+		default:
+			freq = "HOURLY";
+			interval = calnote->recurrence;
+			break;
+		}
+		if (calnote->type == GN_CALNOTE_BIRTHDAY)
+			snprintf(rrule, sizeof(rrule), "FREQ=YEARLY;INTERVAL=1;BYMONTH=%d", stime.month);
+		else if (calnote->occurrences == 0)
+			snprintf(rrule, sizeof(rrule), "FREQ=%s;INTERVAL=%d", freq, interval);
+		else
+			snprintf(rrule, sizeof(rrule), "FREQ=%s;COUNT=%d;INTERVAL=%d", freq, calnote->occurrences, interval);
+		icalcomponent_add_property(vevent, icalproperty_new_rrule(icalrecurrencetype_from_string(rrule)));
+	}
+norecurrence:
+
+	if (calnote->alarm.enabled) {
+		icalcomponent *valarm;
+		struct icaltriggertype trig;
+
+		trig.time.year = (calnote->alarm.timestamp.year == 0xffff ? 1800 : calnote->alarm.timestamp.year);
+		trig.time.month = calnote->alarm.timestamp.month;
+		trig.time.day = calnote->alarm.timestamp.day;
+		trig.time.hour = calnote->alarm.timestamp.hour;
+		trig.time.minute = calnote->alarm.timestamp.minute;
+		trig.time.second = calnote->alarm.timestamp.second;
+		trig.time.is_daylight = 1;
+
+		valarm = icalcomponent_new_valarm();
+
+		icalcomponent_add_property(valarm, icalproperty_new_trigger(trig));
+		/* FIXME: with ICAL_ACTION_DISPLAY a DESCRIPTION property is mandatory */
+		icalcomponent_add_property(valarm, icalproperty_new_action(calnote->alarm.tone ? ICAL_ACTION_AUDIO : ICAL_ACTION_DISPLAY));
+
+		icalcomponent_add_component(vevent, valarm);
+	}
 
 	pIcal = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT,
 				    icalproperty_new_version("2.0"),
 				    icalproperty_new_prodid(get_prodid()),
-				    icalcomponent_vanew(ICAL_VEVENT_COMPONENT,
-							icalproperty_new_dtstart(stime),
-							icalproperty_new_uid(compuid),
-							icalproperty_new_categories("GNOKII"),
-							properties[0],
-							properties[1],
-							properties[2],
-							properties[3],
-							properties[4],
-							0),
+				    vevent,
 				    0);
 
 	if (pIcal) {
@@ -189,7 +266,7 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 
 			icalstr = icalstrbuf;
 		}
-		fprintf(f, "%s\n", icalstr);
+		ical_append_printf(&str, "%s\n", icalstr);
 		dprintf("%s\n", icalstr);
 		if (icalstrbuf)
 			free(icalstrbuf);
@@ -197,50 +274,52 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 		icalcomponent_free(pIcal);
 		pIcal = NULL;
 	} else {
-		dprintf("ERROR in icalcomponent_vanew()\n");
+		dprintf("ERROR in icalcomponent_vanew() creating VCALENDAR\n");
 	}
-	return GN_ERR_NONE;
+	return str.str;
 #else
-	fprintf(f, "BEGIN:VCALENDAR\r\n");
-	fprintf(f, "VERSION:1.0\r\n");
-	fprintf(f, "BEGIN:VEVENT\r\n");
-	fprintf(f, "CATEGORIES:");
+	memset(&str, 0, sizeof (str));
+
+	ical_append_printf(&str, "BEGIN:VCALENDAR\r\n");
+	ical_append_printf(&str, "VERSION:1.0\r\n");
+	ical_append_printf(&str, "BEGIN:VEVENT\r\n");
+	ical_append_printf(&str, "CATEGORIES:");
 	switch (calnote->type) {
 	case GN_CALNOTE_MEMO:
-		fprintf(f, "MISCELLANEOUS\r\n");
+		ical_append_printf(&str, "MISCELLANEOUS\r\n");
 		break;
 	case GN_CALNOTE_REMINDER:
-		fprintf(f, "REMINDER\r\n");
+		ical_append_printf(&str, "REMINDER\r\n");
 		break;
 	case GN_CALNOTE_CALL: 
-		fprintf(f, "PHONE CALL\r\n");
-		fprintf(f, "SUMMARY:%s\r\n", calnote->phone_number);
-		fprintf(f, "DESCRIPTION:%s\r\n", calnote->text);
+		ical_append_printf(&str, "PHONE CALL\r\n");
+		ical_append_printf(&str, "SUMMARY:%s\r\n", calnote->phone_number);
+		ical_append_printf(&str, "DESCRIPTION:%s\r\n", calnote->text);
 		break; 
 	case GN_CALNOTE_MEETING: 
-		fprintf(f, "MEETING\r\n"); 
+		ical_append_printf(&str, "MEETING\r\n"); 
 		if (calnote->mlocation[0])
-			fprintf(f, "LOCATION:%s\r\n", calnote->mlocation);
+			ical_append_printf(&str, "LOCATION:%s\r\n", calnote->mlocation);
 		break; 
 	case GN_CALNOTE_BIRTHDAY: 
-		fprintf(f, "SPECIAL OCCASION\r\n"); 
+		ical_append_printf(&str, "SPECIAL OCCASION\r\n"); 
 		break; 
 	default: 
-		fprintf(f, "UNKNOWN\r\n"); 
+		ical_append_printf(&str, "UNKNOWN\r\n"); 
 		break; 
 	} 
 	if (calnote->type != GN_CALNOTE_CALL)
-		fprintf(f, "SUMMARY:%s\r\n", calnote->text);
-	fprintf(f, "DTSTART:%04d%02d%02dT%02d%02d%02d\r\n", calnote->time.year,
+		ical_append_printf(&str, "SUMMARY:%s\r\n", calnote->text);
+	ical_append_printf(&str, "DTSTART:%04d%02d%02dT%02d%02d%02d\r\n", calnote->time.year,
 		calnote->time.month, calnote->time.day, calnote->time.hour, 
 		calnote->time.minute, calnote->time.second); 
 	if (calnote->end_time.year) {
-		fprintf(f, "DTEND:%04d%02d%02dT%02d%02d%02d\r\n", calnote->end_time.year, 
+		ical_append_printf(&str, "DTEND:%04d%02d%02dT%02d%02d%02d\r\n", calnote->end_time.year, 
 			calnote->end_time.month, calnote->end_time.day, calnote->end_time.hour, 
 			calnote->end_time.minute, calnote->end_time.second); 
 	}
 	if (calnote->alarm.enabled) {
-		fprintf(f, "%sALARM:%04d%02d%02dT%02d%02d%02d\r\n",
+		ical_append_printf(&str, "%sALARM:%04d%02d%02dT%02d%02d%02d\r\n",
 		(calnote->alarm.tone ? "A" : "D"),
 		calnote->alarm.timestamp.year, 
 		calnote->alarm.timestamp.month, calnote->alarm.timestamp.day, calnote->alarm.timestamp.hour, 
@@ -250,30 +329,55 @@ GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
 	case GN_CALNOTE_NEVER: 
 		break; 
 	case GN_CALNOTE_DAILY: 
-		fprintf(f, "RRULE:FREQ=DAILY\r\n"); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=DAILY\r\n") :
+			ical_append_printf(&str, "RRULE:FREQ=DAILY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_WEEKLY: 
-		fprintf(f, "RRULE:FREQ=WEEKLY\r\n"); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=WEEKLY\r\n") :
+			ical_append_printf(&str, "RRULE:FREQ=WEEKLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_2WEEKLY: 
-		fprintf(f, "RRULE:FREQ=WEEKLY;INTERVAL=2\r\n"); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=WEEKLY;INTERVAL=2\r\n") :
+			ical_append_printf(&str, "RRULE:FREQ=WEEKLY;INTERVAL=2;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_MONTHLY: 
-		fprintf(f, "RRULE:FREQ=MONTHLY\r\n"); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=MONTHLY\r\n") :
+			ical_append_printf(&str, "RRULE:FREQ=MONTHLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	case GN_CALNOTE_YEARLY: 
-		fprintf(f, "RRULE:FREQ=YEARLY\r\n"); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=YEARLY\r\n") :
+			ical_append_printf(&str, "RRULE:FREQ=YEARLY;COUNT=%d\r\n", calnote->occurrences);
 		break; 
 	default: 
-		fprintf(f, "RRULE:FREQ=HOURLY;INTERVAL=%d\r\n", calnote->recurrence); 
+		calnote->occurrences ?
+			ical_append_printf(&str, "RRULE:FREQ=HOURLY;INTERVAL=%d\r\n", calnote->recurrence) :
+			ical_append_printf(&str, "RRULE:FREQ=HOURLY;INTERVAL=%d;COUNT=%d\r\n", calnote->recurrence, calnote->occurrences);
 		break; 
 	} 
-	fprintf(f, "END:VEVENT\r\n"); 
-	fprintf(f, "END:VCALENDAR\r\n");
-	return GN_ERR_NONE;
+	ical_append_printf(&str, "END:VEVENT\r\n"); 
+	ical_append_printf(&str, "END:VCALENDAR\r\n");
+	return str.str;
 #endif /* HAVE_LIBICAL */
 }
 
+GNOKII_API int gn_calnote2ical(FILE *f, gn_calnote *calnote)
+{
+	char *vcal;
+	int retval;
+
+	vcal = gn_calnote2icalstr(calnote);
+	if (vcal == NULL)
+		return -1;
+	retval = fputs(vcal, f);
+	free (vcal);
+	return retval;
+}
+
 static inline gn_calnote_type str2calnote_type(const char *str)
 {
 	if (!str) {
@@ -290,27 +394,12 @@ static inline gn_calnote_type str2calnote_type(const char *str)
 	return GN_CALNOTE_REMINDER;
 }
 
-/* read a vcalendar event given by id from file f and store the data in calnote */
-GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
+#ifdef HAVE_LIBICAL
+static int gn_ical2calnote_real(icalcomponent *comp, gn_calnote *calnote, int id)
 {
 	int retval = GN_ERR_FAILED;
-#ifdef HAVE_LIBICAL
-	icalparser *parser = NULL;
-	icalcomponent *comp = NULL, *compresult = NULL;
 	struct icaltimetype dtstart = {0}, dtend = {0};
-
-	parser = icalparser_new();
-	if (!parser) {
-		return GN_ERR_FAILED;
-	}
-
-	icalparser_set_gen_data(parser, f);
-
-	comp = icalparser_parse(parser, ical_fgets);
-	if (!comp) {
-		icalparser_free(parser);
-		return GN_ERR_FAILED;
-	}
+	icalcomponent *compresult = NULL;
 
 	if (id < 1)
 		id = 1;
@@ -324,7 +413,10 @@ GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 	} else {
 		const char *str;
 		icalcomponent *alarm = {0};
+		icalproperty *rrule;
+		struct icalrecurrencetype recur;
 		retval = GN_ERR_NONE;
+
 		calnote->phone_number[0] = 0;
 
 		/* type */
@@ -377,15 +469,16 @@ GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 		}
 
 		/* alarm */
-		alarm = icalcomponent_get_first_component(comp, ICAL_VALARM_COMPONENT);
+		alarm = icalcomponent_get_first_component(compresult, ICAL_VALARM_COMPONENT);
 		if (alarm) {
-			struct icaldurationtype trigger_duration = {0};
-			struct icaltriggertype trigger_value = {0};
 			icalproperty *trigger = NULL;
-			struct icaltimetype alarm_start = {0};
 
 			trigger = icalcomponent_get_first_property(alarm, ICAL_TRIGGER_PROPERTY);
 			if (trigger) {
+				struct icaltriggertype trigger_value;
+				struct icaltimetype alarm_start;
+				icalproperty *property;
+
 				trigger_value = icalvalue_get_trigger(icalproperty_get_value(trigger));
 				if (icaltriggertype_is_null_trigger(trigger_value) ||
 						icaltriggertype_is_bad_trigger(trigger_value)) {
@@ -408,36 +501,159 @@ GNOKII_API int gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
 					calnote->alarm.timestamp.minute = alarm_start.minute;
 					calnote->alarm.timestamp.second = alarm_start.second;
 					/* TODO: time zone? */
+
+					property = icalcomponent_get_first_property(alarm, ICAL_ACTION_PROPERTY);
+					calnote->alarm.tone = icalproperty_get_action(property) == ICAL_ACTION_AUDIO ? 1 : 0;
 				}
 			}
 		}
 
+		/* recurrence */
+		rrule = icalcomponent_get_first_property(compresult, ICAL_RRULE_PROPERTY);
+		if (rrule) {
+			recur = icalproperty_get_rrule(rrule);
+			calnote->occurrences = recur.count;
+			switch (recur.freq) {
+			case ICAL_SECONDLY_RECURRENCE:
+				dprintf("Not suppported recurrence type. Approximating recurrence\n");
+				calnote->recurrence = recur.interval / 3600;
+				if (!calnote->recurrence)
+					calnote->recurrence = 1;
+				break;
+			case ICAL_MINUTELY_RECURRENCE:
+				dprintf("Not suppported recurrence type. Approximating recurrence\n");
+				calnote->recurrence = recur.interval / 60;
+				if (!calnote->recurrence)
+					calnote->recurrence = 1;
+				break;
+			case ICAL_HOURLY_RECURRENCE:
+				calnote->recurrence = recur.interval;
+				break;
+			case ICAL_DAILY_RECURRENCE:
+				calnote->recurrence = recur.interval * 24;
+				break;
+			case ICAL_WEEKLY_RECURRENCE:
+				calnote->recurrence = recur.interval * 24 * 7;
+				break;
+			case ICAL_MONTHLY_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_MONTHLY;
+				break;
+			case ICAL_YEARLY_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_YEARLY;
+				break;
+			case ICAL_NO_RECURRENCE:
+				calnote->recurrence = GN_CALNOTE_NEVER;
+				break;
+			default:
+				dprintf("Not supported recurrence type. Assuming never\n");
+				calnote->recurrence = GN_CALNOTE_NEVER;
+				break;
+			}
+		}
+
 		str = icalcomponent_get_location(compresult);
-		if (!str) str = "";
+		if (!str)
+			str = "";
 		snprintf(calnote->mlocation, sizeof(calnote->mlocation), "%s", str);
-
-		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
-
 	}
-	if (compresult)
+	if (compresult) {
+#ifdef DEBUG
+		char *temp;
+
+		temp = icalcomponent_as_ical_string(compresult);
+		dprintf("Component found\n%s\n", temp);
+		free(temp);
+#endif
 		icalcomponent_free(compresult);
-	if (comp)
-		icalcomponent_free(comp);
-	if (parser)
-		icalparser_free(parser);
-#else
-	retval = GN_ERR_NOTIMPLEMENTED;
-#endif /* HAVE_LIBICAL */
+	}
+
 	return retval;
 }
+#endif /* HAVE_LIBICAL */
+
+/* read a vcalendar event given by id from file f and store the data in calnote */
+GNOKII_API gn_error gn_ical2calnote(FILE *f, gn_calnote *calnote, int id)
+{
+#ifdef HAVE_LIBICAL
+	icalparser *parser = NULL;
+	icalcomponent *comp = NULL;
+	gn_error retval;
+
+	parser = icalparser_new();
+	if (!parser) {
+		return GN_ERR_FAILED;
+	}
+
+	icalparser_set_gen_data(parser, f);
+
+	comp = icalparser_parse(parser, ical_fgets);
+	if (!comp) {
+		icalparser_free(parser);
+		return GN_ERR_FAILED;
+	}
+
+	retval = gn_ical2calnote_real(comp, calnote, id);
+
+	icalcomponent_free(comp);
+	icalparser_free(parser);
+
+	return retval;
+#else
+	return GN_ERR_NOTIMPLEMENTED;
+#endif /* HAVE_LIBICAL */
+}
+
+/* read a vcalendar event given by id from string ical and store the data in calnote */
+GNOKII_API gn_error gn_icalstr2calnote(const char * ical, gn_calnote *calnote, int id)
+{
+#ifdef HAVE_LIBICAL
+	icalparser *parser = NULL;
+	icalcomponent *comp = NULL;
+	gn_error retval;
+
+	parser = icalparser_new();
+	if (!parser) {
+		return GN_ERR_FAILED;
+	}
+
+	comp = icalparser_parse_string (ical);
+	if (!comp) {
+		icalparser_free(parser);
+		return GN_ERR_FAILED;
+	}
+
+	retval = gn_ical2calnote_real(comp, calnote, id);
+
+	icalcomponent_free(comp);
+	icalparser_free(parser);
+
+	return retval;
+#else
+	return GN_ERR_NOTIMPLEMENTED;
+#endif /* HAVE_LIBICAL */
+}
 
 GNOKII_API int gn_todo2ical(FILE *f, gn_todo *ctodo)
+{
+	char *icalstr;
+
+	icalstr = gn_todo2icalstr(ctodo);
+	if (icalstr) {
+		fprintf(f, "%s\n", icalstr);
+		dprintf("%s\n", icalstr);
+		free(icalstr);
+	}
+
+	return GN_ERR_NONE;
+}
+
+GNOKII_API char * gn_todo2icalstr(gn_todo *ctodo)
 {
 #ifdef HAVE_LIBICAL
 	icalcomponent *pIcal = NULL;
 	char compuid[64];
 
-	sprintf(compuid, "guid.gnokii.org_%d_%d", ctodo->location, rand());
+	snprintf(compuid, sizeof(compuid), "guid.gnokii.org_%d_%d", ctodo->location, rand());
 
 	pIcal = icalcomponent_vanew(ICAL_VCALENDAR_COMPONENT,
 				    icalproperty_new_version("2.0"),
@@ -457,36 +673,68 @@ GNOKII_API int gn_todo2ical(FILE *f, gn_todo *ctodo)
 			int icalstrlen = strlen(icalstr);
 			icalstrbuf = malloc(icalstrlen * 2 + 1);
 			utf8_encode(icalstrbuf, icalstrlen * 2, icalstr, icalstrlen);
-			icalstr = icalstrbuf;
+		} else {
+			icalstrbuf = strdup (icalstr);
 		}
-		fprintf(f, "%s\n", icalstr);
-		dprintf("%s\n", icalstr);
-		if (icalstrbuf)
-			free(icalstrbuf);
-
 		icalcomponent_free(pIcal);
-		pIcal = NULL;
+		return icalstrbuf;
 	}
-	return GN_ERR_NONE;
+	return NULL;
 #else
-	fprintf(f, "BEGIN:VCALENDAR\r\n");
-	fprintf(f, "VERSION:1.0\r\n"); 
-	fprintf(f, "BEGIN:VTODO\r\n"); 
-	fprintf(f, "PRIORITY:%i\r\n", ctodo->priority); 
-	fprintf(f, "SUMMARY:%s\r\n", ctodo->text); 
-	fprintf(f, "END:VTODO\r\n"); 
-	fprintf(f, "END:VCALENDAR\r\n");
-	return GN_ERR_NONE;
+	ical_string str;
+
+	memset(&str, 0, sizeof(str));
+	ical_append_printf(&str, "BEGIN:VCALENDAR\r\n");
+	ical_append_printf(&str, "VERSION:1.0\r\n"); 
+	ical_append_printf(&str, "BEGIN:VTODO\r\n"); 
+	ical_append_printf(&str, "PRIORITY:%i\r\n", ctodo->priority); 
+	ical_append_printf(&str, "SUMMARY:%s\r\n", ctodo->text); 
+	ical_append_printf(&str, "END:VTODO\r\n"); 
+	ical_append_printf(&str, "END:VCALENDAR\r\n");
+
+	return str.str;
 #endif /* HAVE_LIBICAL */
 }
 
+#ifdef HAVE_LIBICAL
 /* read the entry identified by id from the vcal file f and write it to the phone */
+static int gn_ical2todo_real(icalcomponent *comp, gn_todo *ctodo, int id)
+{
+	icalcomponent *compresult = NULL;
+
+	if (id < 1)
+		id = 1;
+
+	/* iterate through the component */
+	iterate_cal(comp, 0, &id, &compresult, ICAL_VTODO_COMPONENT);
+
+	if (!compresult) {
+		dprintf("No component found.\n");
+		return GN_ERR_EMPTYLOCATION;
+	} else {
+		icalproperty *priority = icalcomponent_get_first_property(compresult, ICAL_PRIORITY_PROPERTY);
+
+		/* summary goes into text */
+		/* TODO: UTF8 --> ascii */
+		snprintf(ctodo->text, GN_TODO_MAX_LENGTH-1, "%s", icalcomponent_get_summary(compresult));
+
+		/* priority */
+		ctodo->priority = icalproperty_get_priority((const icalproperty *)priority);
+
+		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
+	}
+	icalcomponent_free(compresult);
+
+	return GN_ERR_NONE;
+}
+#endif /* HAVE_LIBICAL */
+
 GNOKII_API int gn_ical2todo(FILE *f, gn_todo *ctodo, int id)
 {
 #ifdef HAVE_LIBICAL
 	icalparser *parser = NULL;
-	icalcomponent *comp = NULL, *compresult = NULL;
-	struct icaltimetype dtstart = {0};
+	icalcomponent *comp = NULL;
+	gn_error error;
 
 	parser = icalparser_new();
 	if (!parser) {
@@ -501,33 +749,41 @@ GNOKII_API int gn_ical2todo(FILE *f, gn_todo *ctodo, int id)
 		return GN_ERR_FAILED;
 	}
 
-	if (id < 1)
-		id = 1;
+	error = gn_ical2todo_real(comp, ctodo, id);
 
-	/* iterate through the component */
-	iterate_cal(comp, 0, &id, &compresult, ICAL_VTODO_COMPONENT);
-
-	if (!compresult) {
-		dprintf("No component found.\n");
-		return GN_ERR_EMPTYLOCATION;
-	} else {
-		const char *priostr = NULL;
-		icalproperty *priority = icalcomponent_get_first_property(compresult, ICAL_PRIORITY_PROPERTY);
-
-		/* summary goes into text */
-		/* TODO: UTF8 --> ascii */
-		snprintf(ctodo->text, GN_CALNOTE_MAX_LENGTH-1, "%s", icalcomponent_get_summary(compresult));
-
-		/* priority */
-		ctodo->priority = icalproperty_get_priority((const icalproperty *)priority);
-
-		dprintf("Component found\n%s\n", icalcomponent_as_ical_string(compresult));
-	}
-	icalcomponent_free(compresult);
 	icalcomponent_free(comp);
 	icalparser_free(parser);
 
-	return GN_ERR_NONE;
+	return error;
+#else
+	return GN_ERR_NOTIMPLEMENTED;
+#endif /* HAVE_LIBICAL */
+}
+
+GNOKII_API int gn_icalstr2todo(const char * ical, gn_todo *ctodo, int id)
+{
+#ifdef HAVE_LIBICAL
+	icalparser *parser = NULL;
+	icalcomponent *comp = NULL;
+	gn_error error;
+
+	parser = icalparser_new();
+	if (!parser) {
+		return GN_ERR_FAILED;
+	}
+
+	comp = icalparser_parse_string (ical);
+	if (!comp) {
+		icalparser_free(parser);
+		return GN_ERR_FAILED;
+	}
+
+	error = gn_ical2todo_real(comp, ctodo, id);
+
+	icalcomponent_free(comp);
+	icalparser_free(parser);
+
+	return error;
 #else
 	return GN_ERR_NOTIMPLEMENTED;
 #endif /* HAVE_LIBICAL */
