@@ -72,7 +72,6 @@
 
 #  include <windows.h>
 #  include <process.h>
-#  include <io.h>
 #  include <getopt.h>
 
 #else
@@ -95,7 +94,6 @@
 
 #include "gnokii-app.h"
 #include "gnokii.h"
-#include <sys/stat.h>
 
 struct gnokii_arg_len {
 	int gal_opt; /* option name (opt_index) */
@@ -225,8 +223,11 @@ static void version(void)
 
 /* The function usage is only informative - it prints this program's usage and
    command-line options. */
-static int usage(FILE *f, int retval)
+static int usage(FILE *f, int argc, char *argv[])
 {
+	int i;
+	int all = 0;
+
 	fprintf(f, _("Usage:\n"
 		     "	gnokii [CONFIG OPTIONS] OPTIONS\n"
 		     "\nCONFIG OPTIONS\n"
@@ -235,25 +236,60 @@ static int usage(FILE *f, int retval)
 		     "\nOPTIONS\n"
 		     "Use just one option at a time.\n"
 		     "General options:\n"
-		     "          --help\n"
+		     "          --help [all] [monitor] [sms] [mms] [phonebook] [calendar] [todo]\n"
+		     "                 [dial] [profile] [settings] [wap] [logo] [ringtone]\n"
+		     "                 [security] [file] [other]\n"
 		     "          --version\n"
 		     "          --shell\n"));
-	monitor_usage(f);
-	sms_usage(f);
-	mms_usage(f);
-	phonebook_usage(f);
-	calendar_usage(f);
-	todo_usage(f);
-	dial_usage(f);
-	profile_usage(f);
-	settings_usage(f);
-	wap_usage(f);
-	logo_usage(f);
-	ringtone_usage(f);
-	security_usage(f);
-	file_usage(f);
-	other_usage(f);
-	return retval;
+	/* Print "short usage" only */
+	if (argc == -1)
+		return -1;
+
+	for (i = 1; i < argc; i++) {
+		if (!strcmp(argv[i], "all")) {
+			all = 1;
+			break;
+		}
+	}
+
+	/* FIXME? throw an error if argv[i] is duplicated or unknown */
+	for (i = 1; i < argc; i++) {
+		if (all || !strcmp(argv[i], "monitor"))
+			monitor_usage(f);
+		if (all || !strcmp(argv[i], "sms"))
+			sms_usage(f);
+		if (all || !strcmp(argv[i], "mms"))
+			mms_usage(f);
+		if (all || !strcmp(argv[i], "phonebook"))
+			phonebook_usage(f);
+		if (all || !strcmp(argv[i], "calendar"))
+			calendar_usage(f);
+		if (all || !strcmp(argv[i], "todo"))
+			todo_usage(f);
+		if (all || !strcmp(argv[i], "dial"))
+			dial_usage(f);
+		if (all || !strcmp(argv[i], "profile"))
+			profile_usage(f);
+		if (all || !strcmp(argv[i], "settings"))
+			settings_usage(f);
+		if (all || !strcmp(argv[i], "wap"))
+			wap_usage(f);
+		if (all || !strcmp(argv[i], "logo"))
+			logo_usage(f);
+		if (all || !strcmp(argv[i], "ringtone"))
+			ringtone_usage(f);
+		/* Unconditional compile here because security_usage() has its own conditional */
+		if (all || !strcmp(argv[i], "security"))
+			security_usage(f);
+		if (all || !strcmp(argv[i], "file"))
+			file_usage(f);
+		if (all || !strcmp(argv[i], "other"))
+			other_usage(f);
+		if (all)
+			break;
+	}
+
+	return 0;
 }
 
 static void gnokii_error_logger(const char *fmt, va_list ap)
@@ -398,7 +434,7 @@ static int parse_options(int argc, char *argv[])
 	static struct option long_options[] = {
 		/* FIXME: these comments are nice, but they would be more useful as docs for the user */
 		/* Display usage. */
-		{ "help",               no_argument,       NULL, OPT_HELP },
+		{ "help",               optional_argument, NULL, OPT_HELP },
 
 		/* Display version and build information. */
 		{ "version",            no_argument,       NULL, OPT_VERSION },
@@ -658,6 +694,7 @@ static int parse_options(int argc, char *argv[])
 	/* Every command which requires arguments should have an appropriate entry
 	   in this array. */
 	static struct gnokii_arg_len gals[] = {
+		{ OPT_HELP,              1, 100, 0 },
 		{ OPT_CONFIGFILE,        1, 100, 0 },
 		{ OPT_CONFIGMODEL,       1, 100, 0 },
 #ifdef SECURITY
@@ -735,7 +772,7 @@ static int parse_options(int argc, char *argv[])
 	switch (c) {
 	/* No argument given - we should display usage. */
 	case -1:
-		return usage(stderr, -1);
+		return usage(stderr, -1, NULL);
 	/* First, error conditions */
 	case '?':
 	case ':':
@@ -746,8 +783,6 @@ static int parse_options(int argc, char *argv[])
 			fprintf(stderr, _("Use '--help' for usage information.\n"));
 		return 1;
 	/* Then, options with no arguments */
-	case OPT_HELP:
-		return usage(stdout, 0);
 	case OPT_VERSION:
 		version();
 		return 0;
@@ -768,21 +803,23 @@ static int parse_options(int argc, char *argv[])
 	   not work as expected; instead args --cmd2 args is passed as a
 	   parameter. */
 	if (checkargs(c, gals, argc, long_options[opt_index].has_arg)) {
-		return usage(stderr, -1);
+		return usage(stderr, -1, NULL);
 	}
 
 	/* Other options that do not need initialization */
 	switch (c) {
 	case OPT_CONFIGFILE:
 		if (configfile)
-			return usage(stderr, -1);
+			return usage(stderr, -1, NULL);
 		configfile = optarg;
 		return parse_options(argc, argv);
 	case OPT_CONFIGMODEL:
 		if (configmodel)
-			return usage(stderr, -1);
+			return usage(stderr, -1, NULL);
 		configmodel = optarg;
 		return parse_options(argc, argv);
+	case OPT_HELP:
+		return usage(stdout, argc, argv);
 	case OPT_VIEWLOGO:
 		return viewlogo(optarg);
 	case OPT_LISTNETWORKS:
@@ -1079,6 +1116,7 @@ int shell(gn_data *data, struct gn_statemachine *state)
 	int len, i, argc = 1;
 	char **argv = NULL;
 	int size = ARGV_CHUNK;
+	int empty = 1;
 
 	argv = calloc(size, sizeof(char *));
 	while (1) {
@@ -1097,16 +1135,20 @@ int shell(gn_data *data, struct gn_statemachine *state)
 			while (*input == ' ')
 				input++;
 			tmp = strstr(input, " ");
-			if (tmp)
+			if (tmp) {
 				len = tmp - input;
-			else
+				*tmp = '\0';
+			} else
 				len = strlen(input);
-			if (len > 0)
+			if (len > 0) {
 				argv[argc++] = strdup(input);
-			input = tmp;
+				empty = 0;
+                        }
+                        input = tmp + (tmp ? 1 : 0);
 		} while (input);
 		argv[argc] = NULL;
-		parse_options(argc, argv);
+		if (!empty)
+        		parse_options(argc, argv);
 		for (i = 1; i < argc; i++)
 			free(argv[i]);
 		free(old);
@@ -1114,7 +1156,7 @@ int shell(gn_data *data, struct gn_statemachine *state)
 	free(argv);
 	return 0;
 #else
-	fprintf(stderr, "gnokii needs to be compiled with getline support to enable this option.\n");
+	fprintf(stderr, _("gnokii needs to be compiled with readline support to enable this option.\n"));
 	return -1;
 #endif
 }
