@@ -1,6 +1,6 @@
 /*
 
-  $Id: gnokii-internal.h,v 1.35 2007/04/14 19:36:35 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -35,7 +35,6 @@
 #ifndef _gnokii_internal_h
 #define _gnokii_internal_h
 
-#include "config.h"
 #include "compat.h"
 #include "misc.h"
 
@@ -45,12 +44,10 @@
 
 #include "gnokii.h"
 
-/* utils */
-gn_connection_type gn_get_connectiontype(const char *connection_type_string);
-
 /* SMS */
-gn_error sms_parse(int offset, gn_data *data);
-gn_error sms_request(gn_data *data, struct gn_statemachine *state);
+gn_error gn_sms_parse(gn_data *data);
+gn_error gn_sms_pdu2raw(gn_sms_raw *rawsms, unsigned char *pdu, int pdu_len, int flags);
+gn_error gn_sms_request(gn_data *data, struct gn_statemachine *state);
 gn_error sms_prepare(gn_sms *sms, gn_sms_raw *rawsms);
 gn_timestamp *sms_timestamp_unpack(unsigned char *number, gn_timestamp *dt);
 unsigned char *sms_timestamp_pack(gn_timestamp *dt, unsigned char *number);
@@ -84,7 +81,7 @@ int char_mblen(const char *src);
 unsigned int char_unicode_decode(unsigned char* dest, const unsigned char* src, int len);
 unsigned int char_unicode_encode(unsigned char* dest, const unsigned char* src, int len);
 
-void char_ascii_decode(unsigned char* dest, const unsigned char* src, int len);
+int char_default_alphabet_decode(unsigned char* dest, const unsigned char* src, int len);
 size_t char_ascii_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
 void char_hex_decode(unsigned char* dest, const unsigned char* src, int len);
@@ -93,8 +90,8 @@ size_t char_hex_encode(char *dest, size_t dest_len, const char *src, size_t len)
 void char_ucs2_decode(unsigned char* dest, const unsigned char* src, int len);
 size_t char_ucs2_encode(char *dest, size_t dest_len, const char *src, size_t len);
 
-unsigned char char_def_alphabet_encode(unsigned char value);
-unsigned char char_def_alphabet_decode(unsigned char value);
+unsigned char char_def_alphabet_encode(unsigned int value);
+unsigned int char_def_alphabet_decode(unsigned char value);
 
 size_t char_uni_alphabet_encode(const char *value, size_t n, wchar_t *dest, MBSTATE *mbs);
 int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs);
@@ -102,6 +99,8 @@ int char_uni_alphabet_decode(wchar_t value, unsigned char *dest, MBSTATE *mbs);
 extern char *char_bcd_number_get(u8 *number);
 extern int char_semi_octet_pack(char *number, unsigned char *output, gn_gsm_number_type type);
 
+int ucs2_encode(char *outstring, int outlen, const char *instring, int inlen);
+
 /* Ringtones */
 int vcal_time_get(gn_timestamp *dt, char *time);
 int calnote_fill(gn_calnote *note, char *type, char *text, char *desc,
@@ -174,6 +173,6 @@ int utf8_base64_decode(char *dest, int destlen, const char *src, int inlen);
 int utf8_base64_encode(char *dest, int destlen, const char *src, int inlen);
 
 int add_slashes(char *dest, char *src, int maxlen, int len);
-int strip_slashes(char *dest, char *src, int maxlen, int len);
+int strip_slashes(char *dest, const char *src, int maxlen, int len);
 
 #endif /* _gnokii_internal_h */
