@@ -1,6 +1,6 @@
 /*
 
-  $Id: common.h,v 1.154 2007/07/09 18:29:00 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -41,7 +41,7 @@
 /* Type of connection. Now we support serial connection with FBUS cable and
    IR (only with 61x0 models) */
 typedef enum {
-        GN_CT_NONE = -1,/* no connection type (error) */
+	GN_CT_NONE = -1,/* no connection type (error) */
 	GN_CT_Serial,   /* Serial connection. */
 	GN_CT_DAU9P,    /* Serial connection using DAU9P cable; use only with 6210/7110 if you want faster initialization */
 	GN_CT_DLR3P,    /* Serial connection using DLR3P cable */
@@ -52,9 +52,12 @@ typedef enum {
 	GN_CT_TCP,      /* TCP network connection */
 	GN_CT_M2BUS,	/* Serial connection with M2BUS protocol */
 	GN_CT_DKU2,	/* DKU2 usb connection using nokia_dku2 kernel driver */
-	GN_CT_DKU2LIBUSB /* DKU2 usb connection using libusb */
+	GN_CT_DKU2LIBUSB, /* DKU2 usb connection using libusb */
+	GN_CT_PCSC      /* PC/SC SIM Card reader using libpsclite */
 } gn_connection_type;
 
+GNOKII_API const char *gn_connection_type2str(gn_connection_type t);
+
 /* Maximum length of device name for serial port */
 
 #define GN_DEVICE_NAME_MAX_LENGTH (32)
@@ -75,13 +78,18 @@ typedef enum {
 	GN_MT_RC, /* Received numbers */
 	GN_MT_MC, /* Missed numbers */
 	GN_MT_LD, /* Last dialed */
+	GN_MT_BD, /* Barred Dialing Numbers */
+	GN_MT_SD, /* Service Dialing Numbers */
 	GN_MT_MT, /* combined ME and SIM phonebook */
 	GN_MT_TA, /* for compatibility only: TA=computer memory */
 	GN_MT_CB, /* Currently selected memory */
 	GN_MT_IN, /* Inbox for folder aware phones */
-	GN_MT_OU, /* Outbox  */
+	GN_MT_OU, /* Outbox, sent items */
 	GN_MT_AR, /* Archive */
 	GN_MT_TE, /* Templates */
+	GN_MT_SR, /* Status reports */
+	GN_MT_DR, /* Drafts */
+	GN_MT_OUS, /* Outbox, items to be sent */
 	GN_MT_F1, /* 1st CUSTOM FOLDER */
 	GN_MT_F2,
 	GN_MT_F3,
@@ -106,12 +114,20 @@ typedef enum {
 	GN_MT_XX = 0xff	/* Error code for unknown memory type (returned by fbus-xxxx functions). */
 } gn_memory_type;
 
-/* Power source types */
+GNOKII_API gn_memory_type gn_str2memory_type(const char *s);
+GNOKII_API const char *gn_memory_type2str(gn_memory_type mt);
+
+/* Power source types - see subclause 8.4 of ETSI TS 127 007 V8.6.0 (2009-01) */
 typedef enum {
-	GN_PS_ACDC = 1, /* AC/DC powered (charging) */
-	GN_PS_BATTERY   /* Internal battery */
+	GN_PS_UNKNOWN,		/* +CME ERROR: ... */
+	GN_PS_ACDC = 1,		/* +CBC: 1,... AC/DC powered (charging) */
+	GN_PS_BATTERY,		/* +CBC: 0,... Battery powered (not charging) */
+	GN_PS_NOBATTERY,	/* +CBC: 2,... No battery */
+	GN_PS_FAULT		/* +CBC: 3,... There is a power fault and calls are inhibited */
 } gn_power_source;
 
+GNOKII_API const char *gn_power_source2str(gn_power_source s);
+
 /* Definition of security codes. */
 typedef enum {
 	GN_SCT_SecurityCode = 0x01, /* Security code. */
@@ -122,6 +138,8 @@ typedef enum {
 	GN_SCT_None                 /* Code not needed. */
 } gn_security_code_type;
 
+GNOKII_API const char *gn_security_code_type2str(gn_security_code_type t);
+
 /* Security code definition. */
 typedef struct {
 	gn_security_code_type type; /* Type of the code. */
@@ -197,6 +215,8 @@ typedef enum {
 	GN_PHONEBOOK_NUMBER_General = 0x0a,
 } gn_phonebook_number_type;
 
+GNOKII_API const char *gn_phonebook_number_type2str(gn_phonebook_number_type t);
+
 typedef enum {
 	GN_PHONEBOOK_ENTRY_Name            = 0x07,
 	GN_PHONEBOOK_ENTRY_Email           = 0x08,
@@ -215,6 +235,7 @@ typedef enum {
 	GN_PHONEBOOK_ENTRY_RingtoneAdv     = 0x37,
 	GN_PHONEBOOK_ENTRY_UserID          = 0x38,
 	GN_PHONEBOOK_ENTRY_PTTAddress      = 0x3f, /* string */
+	GN_PHONEBOOK_ENTRY_ExtGroup        = 0x43,
 	GN_PHONEBOOK_ENTRY_FirstName       = 0x46,
 	GN_PHONEBOOK_ENTRY_LastName        = 0x47,
 	GN_PHONEBOOK_ENTRY_PostalAddress   = 0x4a,
@@ -231,6 +252,9 @@ typedef enum {
 	GN_PHONEBOOK_ENTRY_Birthday        = 0x57,
 } gn_phonebook_entry_type;
 
+GNOKII_API const char *gn_phonebook_entry_type2str(gn_phonebook_entry_type t);
+GNOKII_API const char *gn_subentrytype2string(gn_phonebook_entry_type entry_type, gn_phonebook_number_type number_type);
+
 typedef enum {
 	GN_PHONEBOOK_GROUP_Family,
 	GN_PHONEBOOK_GROUP_Vips,
@@ -240,6 +264,8 @@ typedef enum {
 	GN_PHONEBOOK_GROUP_None,
 } gn_phonebook_group_type;
 
+GNOKII_API const char *gn_phonebook_group_type2str(gn_phonebook_group_type t);
+
 #define GN_PHONEBOOK_PERSON_MAX_LENGTH 64
 
 typedef struct {
@@ -276,7 +302,10 @@ typedef struct {
 		/* GN_PHONEBOOK_ENTRY_Image
 		 * GN_PHONEBOOK_ENTRY_RingtoneAdv
 		 */
-		char fileid[6];                                  /* Bitmap or rigntone fileid */
+		char fileid[6];                                  /* Bitmap or ringtone fileid */
+		/* GN_PHONEBOOK_ENTRY_ExtGroup
+		 */
+		int id;                                          /* Any number, like ID */
 	} data;
 	int id;
 } gn_phonebook_subentry;
@@ -331,7 +360,6 @@ typedef enum {
 } gn_rf_unit;
 
 /* Define enums for Battery units. */
-
 typedef enum {
 	GN_BU_Arbitrary,
 	GN_BU_Volts,
@@ -348,6 +376,8 @@ typedef enum {
 	GN_CALNOTE_MEMO     = 0x16, /* Memo */
 } gn_calnote_type;
 
+GNOKII_API const char *gn_calnote_type2str(gn_calnote_type t);
+
 typedef enum {
 	GN_CALNOTE_NEVER   = 0,
 	GN_CALNOTE_DAILY   = 24,
@@ -357,6 +387,8 @@ typedef enum {
 	GN_CALNOTE_YEARLY  = 65535
 } gn_calnote_recurrence;
 
+GNOKII_API const char *gn_calnote_recurrence2str(gn_calnote_recurrence r);
+
 #define GN_CALNOTE_MAX_NUMBER       1024 /* FIXME how many are possible? */
 #define GN_CALNOTE_MAX_LENGTH        258
 #define GN_CALNOTE_NUMBER_MAX_LENGTH  49
@@ -398,6 +430,8 @@ typedef enum {
 	GN_TODO_HIGH = 1
 } gn_todo_priority;
 
+GNOKII_API const char *gn_todo_priority2str(gn_todo_priority p);
+
 typedef struct {
 	int location;			/* The number of the note in the phone memory */
 	char text[GN_TODO_MAX_LENGTH];		/* The text of the note */
@@ -432,11 +466,15 @@ typedef enum {
 	GN_WAP_SESSION_PERMANENT
 } gn_wap_session;
 
+GNOKII_API const char *gn_wap_session2str(gn_wap_session p);
+
 typedef enum {
 	GN_WAP_AUTH_NORMAL = 0,
 	GN_WAP_AUTH_SECURE
 } gn_wap_authentication;
 
+GNOKII_API const char *gn_wap_authentication2str(gn_wap_authentication p);
+
 typedef enum {
 	GN_WAP_BEARER_GSMDATA = 1,
 	GN_WAP_BEARER_GPRS    = 3,
@@ -444,27 +482,37 @@ typedef enum {
 	GN_WAP_BEARER_USSD    = 9 /* FIXME real value? */
 } gn_wap_bearer;
 
+GNOKII_API const char *gn_wap_bearer2str(gn_wap_bearer p);
+
 typedef enum {
 	GN_WAP_CALL_ANALOGUE,
 	GN_WAP_CALL_ISDN
 } gn_wap_call_type;
 
+GNOKII_API const char *gn_wap_call_type2str(gn_wap_call_type p);
+
 typedef enum {
 	GN_WAP_CALL_AUTOMATIC,
 	GN_WAP_CALL_9600,
 	GN_WAP_CALL_14400
 } gn_wap_call_speed;
 
+GNOKII_API const char *gn_wap_call_speed2str(gn_wap_call_speed p);
+
 typedef enum {
 	GN_WAP_LOGIN_MANUAL,
 	GN_WAP_LOGIN_AUTOLOG
 } gn_wap_login;
 
+GNOKII_API const char *gn_wap_login2str(gn_wap_login p);
+
 typedef enum {
 	GN_WAP_GPRS_ALWAYS,
 	GN_WAP_GPRS_WHENNEEDED
 } gn_wap_gprs;
 
+GNOKII_API const char *gn_wap_gprs2str(gn_wap_gprs p);
+
 typedef struct {
 	int read_before_write;
 	int location;
@@ -539,16 +587,22 @@ typedef enum {
 	GN_PROFILE_MESSAGE_Ascending	= 0x04
 } gn_profile_message_type;
 
+GNOKII_API const char *gn_profile_message_type2str(gn_profile_message_type p);
+
 typedef enum {
 	GN_PROFILE_WARNING_Off		= 0xff,
 	GN_PROFILE_WARNING_On		= 0x04
 } gn_profile_warning_type;
 
+GNOKII_API const char *gn_profile_warning_type2str(gn_profile_warning_type p);
+
 typedef enum {
 	GN_PROFILE_VIBRATION_Off	= 0x00,
 	GN_PROFILE_VIBRATION_On		= 0x01
 } gn_profile_vibration_type;
 
+GNOKII_API const char *gn_profile_vibration_type2str(gn_profile_vibration_type p);
+
 typedef enum {
 	GN_PROFILE_CALLALERT_Ringing		= 0x01,
 	GN_PROFILE_CALLALERT_BeepOnce		= 0x02,
@@ -558,6 +612,8 @@ typedef enum {
 	GN_PROFILE_CALLALERT_CallerGroups	= 0x07
 } gn_profile_callalert_type;
 
+GNOKII_API const char *gn_profile_callalert_type2str(gn_profile_callalert_type p);
+
 typedef enum {
 	GN_PROFILE_KEYVOL_Off		= 0xff,
 	GN_PROFILE_KEYVOL_Level1	= 0x00,
@@ -565,6 +621,8 @@ typedef enum {
 	GN_PROFILE_KEYVOL_Level3	= 0x02
 } gn_profile_keyvol_type;
 
+GNOKII_API const char *gn_profile_keyvol_type2str(gn_profile_keyvol_type p);
+
 typedef enum {
 	GN_PROFILE_VOLUME_Level1	= 0x06,
 	GN_PROFILE_VOLUME_Level2	= 0x07,
@@ -573,6 +631,8 @@ typedef enum {
 	GN_PROFILE_VOLUME_Level5	= 0x0a,
 } gn_profile_volume_type;
 
+GNOKII_API const char *gn_profile_volume_type2str(gn_profile_volume_type p);
+
 /* Structure to hold profile entries. */
 typedef struct {
 	int number;           /* The number of the profile. */
@@ -617,13 +677,16 @@ typedef struct {
 
 /* Data structures for the call divert */
 typedef enum {
-	GN_CDV_Busy = 0x01,
+	GN_CDV_Unconditional = 0x00,
+	GN_CDV_Busy,
 	GN_CDV_NoAnswer,
 	GN_CDV_OutOfReach,
 	GN_CDV_NotAvailable,
 	GN_CDV_AllTypes
 } gn_call_divert_type;
 
+GNOKII_API const char *gn_call_divert_type2str(gn_call_divert_type p);
+
 typedef enum {
 	GN_CDV_VoiceCalls = 0x01,
 	GN_CDV_FaxCalls,
@@ -631,6 +694,8 @@ typedef enum {
 	GN_CDV_AllCalls
 } gn_call_divert_call_type;
 
+GNOKII_API const char *gn_call_divert_call_type2str(gn_call_divert_call_type p);
+
 typedef enum {
 	GN_CDV_Disable  = 0x00,
 	GN_CDV_Enable   = 0x01,
@@ -639,6 +704,8 @@ typedef enum {
 	GN_CDV_Erasure  = 0x04
 } gn_call_divert_operation;
 
+GNOKII_API const char *gn_call_divert_operation2str(gn_call_divert_operation p);
+
 typedef struct {
 	gn_call_divert_type           type;
 	gn_call_divert_call_type     ctype;
@@ -797,12 +864,11 @@ typedef struct {
 	unsigned char *file;	/* file contents */
 } gn_file;
 
-#define GN_FILES_MAX_COUNT 1024
-
 typedef struct {
 	char path[512];
-	gn_file *files[GN_FILES_MAX_COUNT];
+	gn_file **files;
 	int file_count;
+	int size;
 } gn_file_list;
 
 #endif	/* _gnokii_common_h */
