@@ -1,6 +1,6 @@
 /*
 
-  $Id: misc.h,v 1.54 2006/12/17 16:02:07 pkot Exp $
+  $Id$
 
   G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999-2000 Hugh Blemings & Pavel Jank ml.
+  Copyright (C) 1999-2000 Hugh Blemings & Pavel Janik ml.
   Copyright (C) 2000      Jan Derfinak
   Copyright (C) 2001-2002 Chris Kemp, Pavel Machek
   Copyright (C) 2001-2004 Pawel Kot, BORBELY Zoltan
@@ -37,46 +37,12 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-#include "config.h"
 #include "compat.h"
 
 /* Some general defines. */
 
-#ifndef false
-#  define false (0)
-#endif
-
-#ifndef true
-#  define true (!false)
-#endif
-
-#ifndef bool
-#  define bool int
-#endif
-
 #define ARRAY_LEN(x) (sizeof((x)) / sizeof((x)[0]))
 
-#define SAFE_STRNCPY(dest, src, n) do { \
-	strncpy((dest), (src), (n)); \
-	if ((n) > 0) \
-		(dest)[(n)-1] = '\0'; \
-	} while (0)
-
-#define SAFE_STRNCPY_SIZEOF(dest,src) \
-	SAFE_STRNCPY((dest), (src), sizeof((dest)))
-
-/* If glib.h is included, G_GNUC_PRINTF is already defined. */
-#ifndef G_GNUC_PRINTF
-/* Stolen from <glib.h>: */
-#  if	__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
-#    define G_GNUC_PRINTF( format_idx, arg_idx )	\
-      __attribute__((format (printf, format_idx, arg_idx)))
-#  else	/* !__GNUC__ */
-#    define G_GNUC_PRINTF( format_idx, arg_idx )
-#  endif	/* !__GNUC__ */
-#endif
-
-
 #define GNOKII_MAX(a, b)  (((a) > (b)) ? (a) : (b))
 #define GNOKII_MIN(a, b)  (((a) < (b)) ? (a) : (b))
 
@@ -126,6 +92,11 @@
 #define PM_AUTHENTICATION	0x0200
 #define PM_FOLDERS		0x0400
 #define PM_FULLPBK		0x0800
+/* Verify if these three might not be coupled into one */
+#define PM_SMSFILE		0x1000
+#define PM_EXTPBK2		0x2000
+#define PM_EXTCALENDAR		0x4000
+
 /* This one indicated reported cases of breaking the phone by xgnokii
  * in FBUS orver IrDA mode */
 #define PM_XGNOKIIBREAKAGE	0x8000
