@@ -1,6 +1,6 @@
 /*
 
-  $Id: xgnokii_lowlevel.c,v 1.98 2006/12/17 17:23:37 pkot Exp $
+  $Id$
   
   X G N O K I I
 
@@ -22,7 +22,7 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999      Pavel Jank ml., Hugh Blemings
+  Copyright (C) 1999      Pavel Janik ml., Hugh Blemings
   Copyright (C) 1999-2005 Jan Derfinak
   Copyright (C) 2002-2004 Pawel Kot, BORBELY Zoltan
   Copyright (C) 2002      Markus Plail, Manfred Jonsson, Pavel Machek
@@ -30,6 +30,8 @@
 
 */
 
+#include "config.h"
+
 #ifndef WIN32
 #  include <unistd.h>
 #endif
@@ -464,11 +466,12 @@ static gint A_GetMemoryLocationAll(gpointer data)
 	gn_data_clear(&gdat);
 
 	error = mla->status = GN_ERR_NONE;
-	entry.memory_type = mla->type;
 	gdat.phonebook_entry = &entry;
 
 	pthread_mutex_lock(&memoryMutex);
 	for (i = mla->min; i <= mla->max; i++) {
+		memset(&entry, 0, sizeof(entry));
+		entry.memory_type = mla->type;
 		entry.location = i;
 		error = gn_sm_functions(GN_OP_ReadPhonebook, &gdat, statemachine);
 		if (error != GN_ERR_NONE && error != GN_ERR_INVALIDLOCATION &&
@@ -504,7 +507,7 @@ static gint A_GetMemoryLocationAll(gpointer data)
 				return GN_ERR_NONE;
 			}
 		}
-		if (error == GN_ERR_NONE) read++;
+		if ((error == GN_ERR_NONE) && (entry.empty == false)) read++;
 		dprintf("Name: %s\n", entry.name);
 		error = mla->InsertEntry(&entry); 
 		/* FIXME: It only works this way at the moment */
@@ -566,6 +569,31 @@ static gint A_WriteMemoryLocationAll(gpointer data)
 	return error;
 }
 
+
+static gint A_DeleteMemoryLocation(gpointer data)
+{
+	gn_error error;
+	D_MemoryLocation *ml = (D_MemoryLocation *) data;
+	gn_data gdat;
+
+	if (!data) return GN_ERR_INTERNALERROR;
+
+	gn_data_clear(&gdat);
+
+	error = ml->status = GN_ERR_UNKNOWN;
+
+	if (ml->entry) {
+		gn_phonebook_entry_sanitize(ml->entry);
+		gdat.phonebook_entry = (ml->entry);
+		pthread_mutex_lock(&memoryMutex);
+		error = ml->status = gn_sm_functions(GN_OP_DeletePhonebook, &gdat, statemachine);
+		pthread_cond_signal(&memoryCond);
+		pthread_mutex_unlock(&memoryMutex);
+	}
+
+	return (error);
+}
+
 static gint A_GetCalendarNote(gpointer data)
 {
 	gn_error error;
@@ -599,6 +627,9 @@ static gint A_GetCalendarNoteAll(gpointer data)
 
 	gn_data_clear(&gdat);
 
+	memset(&entry, 0, sizeof(entry));
+	memset(&list, 0, sizeof(list));
+
 	pthread_mutex_lock(&calendarMutex);
 	while (1) {
 		entry.location = i;
@@ -767,6 +798,7 @@ static gint A_SendSMSMessage(gpointer data)
 		gdat.sms = d->sms;
 		pthread_mutex_lock(&sendSMSMutex);
 		error = d->status = gn_sms_send(&gdat, statemachine);
+		free(gdat.sms->reference);
 		pthread_cond_signal(&sendSMSCond);
 		pthread_mutex_unlock(&sendSMSMutex);
 	}
@@ -1032,6 +1064,8 @@ static gint A_GetNetworkInfo(gpointer data)
 
 	pthread_mutex_lock(&getNetworkInfoMutex);
 	gdat.network_info = d->info;
+	gdat.reg_notification = NULL;
+	gdat.callback_data = NULL;
 	error = d->status = gn_sm_functions(GN_OP_GetNetworkInfo, &gdat, statemachine);
 	pthread_cond_signal(&getNetworkInfoCond);
 	pthread_mutex_unlock(&getNetworkInfoMutex);
@@ -1165,6 +1199,7 @@ gint(*DoAction[])(gpointer) = {
 	    A_GetMemoryLocationAll,
 	    A_WriteMemoryLocation,
 	    A_WriteMemoryLocationAll,
+	    A_DeleteMemoryLocation,
 	    A_GetCalendarNote,
 	    A_GetCalendarNoteAll,
 	    A_WriteCalendarNote,
@@ -1217,6 +1252,7 @@ void *GUI_Connect(void *a)
 	if (fbusinit() != GN_ERR_NONE) {
 		gn_log_xdebug("Initialization failed...\n");
 		MainExit(NULL);
+		return NULL;
 	}
 
 	if (!xgnokiiConfig.allowBreakage &&				/* User did not allow to break the phone */
@@ -1229,6 +1265,7 @@ void *GUI_Connect(void *a)
 			   "\tallow_breakage = 1\n\n"
 			   "in xgnokii section in your config file.\n"
 			   "See http://thread.gmane.org/gmane.linux.drivers.gnokii/3195 for details.\n"));
+		return NULL;
 	}
 
 	gn_log_xdebug("Phone connected. Starting monitoring...\n");
