@@ -1,10 +1,10 @@
 /*
 
-  $Id: xgnokii_sms.c,v 1.57 2004/02/20 11:02:26 uid66849 Exp $
+  $Id$
 
   X G N O K I I
 
-  A Linux/Unix GUI for Nokia mobile phones.
+  A Linux/Unix GUI for the mobile phones.
 
   This file is part of gnokii.
 
@@ -22,11 +22,20 @@
   along with gnokii; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-  Copyright (C) 1999 Pavel Jank ml., Hugh Blemings
-  & Jn Derfik <ja@mail.upjs.sk>.
+  Copyright (C) 1999 Pavel Janik ml., Hugh Blemings
+  Copyright (C) 1999-2005 Jan Derfinak
+  Copyright (C) 2001-2003 Pawel Kot
+  Copyright (C) 2002-2003 BORBELY Zoltan
+  Copyright (C) 2002      Markus Plail
+  Copyright (C) 2003      Uli Hopp
 
 */
-#include <unistd.h>
+
+#include "config.h"
+
+#ifndef WIN32
+#  include <unistd.h>
+#endif
 #include <locale.h>
 #include <fcntl.h>
 #include <pthread.h>
@@ -104,22 +113,6 @@ static InfoDialog infoDialog = { NULL, NULL };
 static QuestMark questMark;
 static GtkWidget *treeFolderItem[GN_SMS_FOLDER_MAX_NUMBER], *subTree;
 
-static inline void Help1(GtkWidget * w, gpointer data)
-{
-	gchar *indx = g_strdup_printf("/help/%s/windows/sms/index.html", xgnokiiConfig.locale);
-	Help(w, indx);
-	g_free(indx);
-}
-
-
-static inline void Help2(GtkWidget * w, gpointer data)
-{
-	gchar *indx = g_strdup_printf("/help/%s/sms_send.html", xgnokiiConfig.locale);
-	Help(w, indx);
-	g_free(indx);
-}
-
-
 static inline void CloseSMS(GtkWidget * w, gpointer data)
 {
 	gtk_widget_hide(GUI_SMSWindow);
@@ -228,7 +221,7 @@ static int IsValidSMSforFolder(gpointer d, gpointer userData)
 			return 1;
 	} else {
 		if ((data->type == GN_SMS_MT_Deliver && !SMS.currentBox) ||
-		    (data->type == GN_SMS_MT_DeliveryReport && !SMS.currentBox) ||
+		    (data->type == GN_SMS_MT_StatusReport && !SMS.currentBox) ||
 		    (data->type == GN_SMS_MT_Submit && SMS.currentBox))
 			return 1;
 	}
@@ -303,7 +296,7 @@ static void InsertFolderElement(gpointer d, gpointer userData)
 		*/
 		gchar *row[4];
 		switch (data->type) {
-		case GN_SMS_MT_DeliveryReport:
+		case GN_SMS_MT_StatusReport:
 			switch (data->status) {
 			case GN_SMS_Read:
 				row[0] = g_strdup(_("read report"));
@@ -353,18 +346,18 @@ static void InsertFolderElement(gpointer d, gpointer userData)
 
 		if (dt) {
 			if (dt->timezone)
-				row[1] = g_strdup_printf("%02d/%02d/%04d %02d:%02d:%02d %c%02d00",
+				row[1] = g_strdup_printf(_("%02d/%02d/%04d %02d:%02d:%02d %c%02d00"),
 							 dt->day, dt->month, dt->year,
 							 dt->hour, dt->minute, dt->second,
 							 dt->timezone > 0 ? '+' : '-',
 							 abs(dt->timezone));
 			else
-				row[1] = g_strdup_printf("%02d/%02d/%04d %02d:%02d:%02d",
+				row[1] = g_strdup_printf(_("%02d/%02d/%04d %02d:%02d:%02d"),
 							 dt->day, dt->month, dt->year,
 							 dt->hour, dt->minute, dt->second);
 		} else {
 			row[1] =
-			    g_strdup_printf("%02d/%02d/%04d %02d:%02d:%02d", 01, 01, 0001, 01, 01,
+			    g_strdup_printf(_("%02d/%02d/%04d %02d:%02d:%02d"), 01, 01, 0001, 01, 01,
 					    01);
 		}
 
@@ -372,7 +365,7 @@ static void InsertFolderElement(gpointer d, gpointer userData)
 		if (row[2] == NULL)
 			row[2] = data->remote.number;
 		if ((data->type == GN_SMS_MT_Picture) || (data->type == GN_SMS_MT_PictureTemplate))
-			row[3] = g_strdup_printf("Picture Message: %s", data->user_data[1].u.text);
+			row[3] = g_strdup_printf(_("Picture Message: %s"), data->user_data[1].u.text);
 		else
 			row[3] = data->user_data[0].u.text;
 
@@ -506,7 +499,7 @@ inline void GUI_ShowSMS(void)
 	for (j = i + 1; j < lastfoldercount; j++)
 		gtk_widget_hide(treeFolderItem[j - 1]);
 	lastfoldercount = foldercount;
-	gtk_widget_show(GUI_SMSWindow);
+	gtk_window_present(GTK_WINDOW(GUI_SMSWindow));
 	GUI_RefreshMessageWindow();
 }
 
@@ -717,7 +710,7 @@ static void SaveToMailbox(gchar *mailbox_name)
 
 static void OKSaveSMStoMailbox(GtkWidget * widget, gpointer data)
 {
-  mailbox_name = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
+  mailbox_name = (gchar *) gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
   SaveToMailbox(mailbox_name);
   gtk_widget_hide(GTK_WIDGET(data));
 }
@@ -758,7 +751,7 @@ create_SaveSMStoMailbox (void)
   SaveSMStoMailbox = gtk_file_selection_new (_("Choose Mailbox File"));
   gtk_object_set_data (GTK_OBJECT (SaveSMStoMailbox), "SaveSMStoMailbox", SaveSMStoMailbox);
   gtk_container_set_border_width (GTK_CONTAINER (SaveSMStoMailbox), 10);
-  GTK_WINDOW (SaveSMStoMailbox)->type = GTK_WINDOW_DIALOG;
+  GTK_WINDOW (SaveSMStoMailbox)->type = GTK_WINDOW_TOPLEVEL;
 
   ok_button1 = GTK_FILE_SELECTION (SaveSMStoMailbox)->ok_button;
   gtk_object_set_data (GTK_OBJECT (SaveSMStoMailbox), "ok_button1", ok_button1);
@@ -806,7 +799,7 @@ static inline void RefreshSMSStatus(void)
 static inline gint RefreshSMSLength(GtkWidget * widget, GdkEventKey * event, gpointer callback_data)
 {
 	RefreshSMSStatus();
-	if (GTK_EDITABLE(widget)->editable == FALSE)
+	if (gtk_editable_get_editable (GTK_EDITABLE(widget)) == FALSE)
 		return (FALSE);
 	if (event->keyval == GDK_BackSpace || event->keyval == GDK_Clear ||
 	    event->keyval == GDK_Insert || event->keyval == GDK_Delete ||
@@ -871,7 +864,7 @@ static inline void InitAddressLine(gpointer d, gpointer userData)
 #ifdef XDEBUG
 static inline void PrintAddressLine(gpointer d, gpointer userData)
 {
-	g_print("Name: %s\nNumber: %s\nUsed: %d\n",
+	g_print(_("Name: %s\nNumber: %s\nUsed: %d\n"),
 		((AddressPar *) d)->name, ((AddressPar *) d)->number, ((AddressPar *) d)->used);
 }
 #endif
@@ -1074,13 +1067,13 @@ static gint Sendsmscore(gn_sms * sms)
 	g_free(m);
 
 	if (error != GN_ERR_NONE) {
-		gchar *buf = g_strdup_printf(_("SMS send to %s failed\n(error=%d)"),
-					     sms->remote.number, error);
+		gchar *buf = g_strdup_printf(_("Error sending SMS to %s\n%s"),
+					     sms->remote.number, gn_error_print(error));
 		gtk_label_set_text(GTK_LABEL(errorDialog.text), buf);
 		gtk_widget_show(errorDialog.dialog);
 		g_free(buf);
 	} else
-		g_print("Message sent to: %s\n", sms->remote.number);
+		g_print(_("Message sent to: %s\n"), sms->remote.number);
 
 	return (error);
 }
@@ -1134,8 +1127,8 @@ static void DoSendSMS(void)
 			sms.delivery_report = false;
 		sms.type = GN_SMS_MT_Submit;
 
-		strncpy(sms.remote.number, number, GN_BCD_STRING_MAX_LENGTH + 1);
-		sms.remote.number[GN_BCD_STRING_MAX_LENGTH] = '\0';
+		strncpy(sms.remote.number, number, GN_BCD_STRING_MAX_LENGTH);
+		sms.remote.number[GN_BCD_STRING_MAX_LENGTH - 1] = '\0';
 		if (sms.remote.number[0] == '+')
 			sms.remote.type = GN_GSM_NUMBER_International;
 		else 
@@ -1218,7 +1211,7 @@ static void DoSendSMS(void)
 			strncpy(sms.user_data[0].u.text, text, GN_SMS_MAX_LENGTH + 1);
 			sms.user_data[0].u.text[GN_SMS_MAX_LENGTH] = '\0';
 
-			buf = g_strdup_printf(_("Sending SMS to %s ...\n"), sms.remote.number);
+			buf = g_strdup_printf(_("Sending SMS to %s (%d/%d) ...\n"), sms.remote.number, 1, 1);
 			gtk_label_set_text(GTK_LABEL(infoDialog.text), buf);
 			gtk_widget_show_now(infoDialog.dialog);
 			g_free(buf);
@@ -1246,24 +1239,22 @@ static GtkItemFactoryEntry send_menu_items[] = {
 	{NULL, NULL, NULL, 0, "<Separator>"},
 	{NULL, "<control>W", CloseSMSSend, 0, NULL},
 	{NULL, NULL, NULL, 0, "<LastBranch>"},
-	{NULL, NULL, Help2, 0, NULL},
 	{NULL, NULL, GUI_ShowAbout, 0, NULL},
 };
 
 
 static void InitSendMenu(void)
 {
-	send_menu_items[0].path = g_strdup(_("/_File"));
-	send_menu_items[1].path = g_strdup(_("/File/Sen_d"));
-	send_menu_items[2].path = g_strdup(_("/File/_Save"));
-	send_menu_items[3].path = g_strdup(_("/File/Sep1"));
-	send_menu_items[4].path = g_strdup(_("/File/Check _Names"));
-	send_menu_items[5].path = g_strdup(_("/File/C_ontacts"));
-	send_menu_items[6].path = g_strdup(_("/File/Sep2"));
-	send_menu_items[7].path = g_strdup(_("/File/_Close"));
-	send_menu_items[8].path = g_strdup(_("/_Help"));
-	send_menu_items[9].path = g_strdup(_("/Help/_Help"));
-	send_menu_items[10].path = g_strdup(_("/Help/_About"));
+	send_menu_items[0].path = _("/_File");
+	send_menu_items[1].path = _("/File/Sen_d");
+	send_menu_items[2].path = _("/File/_Save");
+	send_menu_items[3].path = _("/File/Sep1");
+	send_menu_items[4].path = _("/File/Check _Names");
+	send_menu_items[5].path = _("/File/C_ontacts");
+	send_menu_items[6].path = _("/File/Sep2");
+	send_menu_items[7].path = _("/File/_Close");
+	send_menu_items[8].path = _("/_Help");
+	send_menu_items[9].path = _("/Help/_About");
 }
 
 
@@ -1301,7 +1292,7 @@ static void CreateSMSSendWindow(void)
 
 	gtk_item_factory_create_items(item_factory, nmenu_items, send_menu_items, NULL);
 
-	gtk_accel_group_attach(accel_group, GTK_OBJECT(sendSMS.SMSSendWindow));
+	gtk_window_add_accel_group(GTK_WINDOW(sendSMS.SMSSendWindow), accel_group);
 
 	/* Finally, return the actual menu bar created by the item factory. */
 	menubar = gtk_item_factory_get_widget(item_factory, "<main>");
@@ -1316,8 +1307,9 @@ static void CreateSMSSendWindow(void)
 
 	/* Create the toolbar */
 
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("Send message"), NULL,
 				NewPixmap(SendSMS_xpm, GUI_SMSWindow->window,
@@ -1455,7 +1447,7 @@ static void NewSMS(void)
 
 	RefreshSMSStatus();
 
-	gtk_widget_show(sendSMS.SMSSendWindow);
+	gtk_window_present(GTK_WINDOW(sendSMS.SMSSendWindow));
 }
 
 
@@ -1489,7 +1481,7 @@ static void ForwardSMS(void)
 
 	RefreshSMSStatus();
 
-	gtk_widget_show(sendSMS.SMSSendWindow);
+	gtk_window_present(GTK_WINDOW(sendSMS.SMSSendWindow));
 }
 
 
@@ -1548,7 +1540,7 @@ static void ReplySMS(void)
 	CheckAddressMain();
 	RefreshSMSStatus();
 
-	gtk_widget_show(sendSMS.SMSSendWindow);
+	gtk_window_present(GTK_WINDOW(sendSMS.SMSSendWindow));
 }
 
 static void ActivateSMS(void)
@@ -1628,7 +1620,7 @@ static void NewBC(void)
 
 	RefreshSMSStatus();
 
-	gtk_widget_show(sendSMS.SMSSendWindow);
+	gtk_window_present(GTK_WINDOW(sendSMS.SMSSendWindow));
 }
 
 
@@ -1640,7 +1632,7 @@ static GtkItemFactoryEntry menu_items[] = {
 	{NULL, NULL, NULL, 0, "<Separator>"},
 	{NULL, "<control>W", CloseSMS, 0, NULL},
 	{NULL, NULL, NULL, 0, "<Branch>"},
-	{NULL, "<control>A", ActivateSMS, 0, NULL},
+	{NULL, "<control>A", ActivateSMS, 0, "<CheckItem>"},
 	{NULL, "<control>N", NewSMS, 0, NULL},
 	{NULL, "<control>F", ForwardSMS, 0, NULL},
 	{NULL, "<control>R", ReplySMS, 0, NULL},
@@ -1648,7 +1640,6 @@ static GtkItemFactoryEntry menu_items[] = {
 	{NULL, NULL, NULL, 0, "<Separator>"},
 	{NULL, "<control>B", NewBC, 0, NULL},
 	{NULL, NULL, NULL, 0, "<LastBranch>"},
-	{NULL, NULL, Help1, 0, NULL},
 	{NULL, NULL, GUI_ShowAbout, 0, NULL},
 };
 
@@ -1657,23 +1648,22 @@ static void InitMainMenu(void)
 {
 	register gint i = 0;
 
-	menu_items[i++].path = g_strdup(_("/_File"));
-	menu_items[i++].path = g_strdup(_("/File/_Save"));
-	menu_items[i++].path = g_strdup(_("/File/Save to mailbo_x"));
-	menu_items[i++].path = g_strdup(_("/File/Save to _file"));
-	menu_items[i++].path = g_strdup(_("/File/Sep1"));
-	menu_items[i++].path = g_strdup(_("/File/_Close"));
-	menu_items[i++].path = g_strdup(_("/_Messages"));
-	menu_items[i++].path = g_strdup(_("/_Messages/(De)_Activate SMS reading"));
-	menu_items[i++].path = g_strdup(_("/_Messages/_New"));
-	menu_items[i++].path = g_strdup(_("/_Messages/_Forward"));
-	menu_items[i++].path = g_strdup(_("/_Messages/_Reply"));
-	menu_items[i++].path = g_strdup(_("/_Messages/_Delete"));
-	menu_items[i++].path = g_strdup(_("/Messages/Sep3"));
-	menu_items[i++].path = g_strdup(_("/_Messages/_Business card"));
-	menu_items[i++].path = g_strdup(_("/_Help"));
-	menu_items[i++].path = g_strdup(_("/Help/_Help"));
-	menu_items[i++].path = g_strdup(_("/Help/_About"));
+	menu_items[i++].path = _("/_File");
+	menu_items[i++].path = _("/File/_Save");
+	menu_items[i++].path = _("/File/Save to mailbo_x");
+	menu_items[i++].path = _("/File/Save to _file");
+	menu_items[i++].path = _("/File/Sep1");
+	menu_items[i++].path = _("/File/_Close");
+	menu_items[i++].path = _("/_Messages");
+	menu_items[i++].path = _("/_Messages/_Activate SMS reading");
+	menu_items[i++].path = _("/_Messages/_New");
+	menu_items[i++].path = _("/_Messages/_Forward");
+	menu_items[i++].path = _("/_Messages/_Reply");
+	menu_items[i++].path = _("/_Messages/_Delete");
+	menu_items[i++].path = _("/Messages/Sep3");
+	menu_items[i++].path = _("/_Messages/_Business card");
+	menu_items[i++].path = _("/_Help");
+	menu_items[i++].path = _("/Help/_About");
 }
 
 
@@ -1710,7 +1700,7 @@ void GUI_CreateSMSWindow(void)
 
 	gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
 
-	gtk_accel_group_attach(accel_group, GTK_OBJECT(GUI_SMSWindow));
+	gtk_window_add_accel_group(GTK_WINDOW(GUI_SMSWindow), accel_group);
 
 	/* Finally, return the actual menu bar created by the item factory. */
 	menubar = gtk_item_factory_get_widget(item_factory, "<main>");
@@ -1724,8 +1714,9 @@ void GUI_CreateSMSWindow(void)
 	gtk_widget_show(menubar);
 
 	/* Create the toolbar */
-	toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-	gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NORMAL);
+	toolbar = gtk_toolbar_new();
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), NULL, _("New message"), NULL,
 				NewPixmap(Edit_xpm, GUI_SMSWindow->window,
@@ -1756,14 +1747,14 @@ void GUI_CreateSMSWindow(void)
 	gtk_widget_show(toolbar);
 
 	vpaned = gtk_vpaned_new();
-	gtk_paned_set_handle_size(GTK_PANED(vpaned), 10);
-	gtk_paned_set_gutter_size(GTK_PANED(vpaned), 15);
+//	gtk_paned_set_handle_size(GTK_PANED(vpaned), 10);
+//	gtk_paned_set_gutter_size(GTK_PANED(vpaned), 15);
 	gtk_box_pack_end(GTK_BOX(main_vbox), vpaned, TRUE, TRUE, 0);
 	gtk_widget_show(vpaned);
 
 	hpaned = gtk_hpaned_new();
-	gtk_paned_set_handle_size(GTK_PANED(hpaned), 8);
-	gtk_paned_set_gutter_size(GTK_PANED(hpaned), 10);
+//	gtk_paned_set_handle_size(GTK_PANED(hpaned), 8);
+//	gtk_paned_set_gutter_size(GTK_PANED(hpaned), 10);
 	gtk_paned_add1(GTK_PANED(vpaned), hpaned);
 	gtk_widget_show(hpaned);
 
